#include <PalmOS.h>
#include "SugarMemo.h"
#include "SugarMemoRsc.h"
#include "SugarMemoBrowse.h"
#include "SugarMemoDB.h"
#include "SugarMemoTools.h"

#define statusColumn 	0
#define entryColumn 	1
#define arrowColumn 	2

#define statusColumnWidth 	11
#define arrowColumnWidth 	13
#define columnSpacing 		2

UInt16 		CurrentCategory = dmAllCategories;
Char 		CategoryName[dmCategoryLength];
UInt16      TopVisibleRecord = 0;
Boolean		ShowAllCategories = true;
Int16		VisibleRows = 0;


//*******************************************************************
//	Internal variables
//*******************************************************************

TabSelected 	tabCurrentSelected = StudyListTab;

UInt16 			TopVisibleBackup[TabCounts] = {0, FirstWordRecord, FirstWordRecord};
const Int16 	RowsInTable = 11;
Header 			HeaderMask;

UInt32			SearchFieldTicks = 0;
Boolean			SearchFieldTicking = false;

extern void EditNewRecord();

//*******************************************************************
//	Internal functions
//*******************************************************************
static void 	BrowseFormLoadTable( FormType* frmP );
static void		WordListDrawRecord (void * table, Int16 row, Int16 column, RectanglePtr bounds);
static void 	BrowseFormInit( FormType* frmP );
static void 	ClearWordListTable();

Boolean 		SeekNextRecord(UInt16 * indexP, Int16 direction);
static void 	WordListForward(Boolean byline);
static void 	WordListBackward(Boolean byline);
static UInt16 	WordListSelectCategory (void);
static void 	ToolsChangeCategory (UInt16 category);
static void 	ChangeTab(TabSelected tab);
static void 	DrawTitleBar();
static void 	MoveToStudyList(Int16 row);
static void 	MoveToRawList(Int16 row);
static void 	BatchMoveToStudyList();
static void 	BatchMoveToRawList();
static void 	UpdateSearchField();
static void 	InstantLookup(EventType * event);
static void 	SearchFieldIdle(FieldType* fldP);
static void 	SortStudyList();
static void 	DetectPenDown(EventType* event);
static void 	ShuffleStudyList();
static Char 	WordQueueToChar(HeaderQueue queue);

/////////////////////////////////////////////////
//
//	event handler
//
/////////////////////////////////////////////////
Boolean BrowseFormHandleEvent(EventType * event){

	FormType* frmP = FrmGetActiveForm();
	FormType* form;
	Boolean handled = false;
	Char* p;
	UInt32	x;
	
	frmP = FrmGetActiveForm();
	
	switch (event->eType){
			
		case frmOpenEvent:
			BrowseFormInit(frmP);
			handled = true;
			break;
												
		case ctlSelectEvent:
			switch (event->data.ctlSelect.controlID){
				case BrowseCategoryPopTrigger:
					WordListSelectCategory();
					handled = true;
					break;
							
				case BrowseQuitButton:
					
					//close database if possible
					if (CurrentDB != NULL) {
						if (errNone != DmCloseDatabase(CurrentDB))
						DoCustomDialog("", "޷رݿ", false, 0, NULL, false);
					}

					FrmGotoForm(MainForm);
					handled = true;
					break;
				
				case BrowseNewButton:
					EditNewRecord();
					handled = true;
					break;

				case BrowseShuffleButton:
					ShuffleStudyList();
					handled = true;
					break;
										
				//case BrowseSortButton:
				//	SortStudyList();					
				//	handled = true;
				//	break;
				
				case BrowseAddAllButton:
					BatchMoveToStudyList();
					handled = true;
					break;
					
				case BrowseRemoveAllButton:
					BatchMoveToRawList();
					handled = true;
					break;
					
				case BrowseStudyListPushButton:
					ChangeTab(StudyListTab);
					break;
				
				case BrowseRawPushButton:
					ChangeTab(RawTab);
					break;
				
				case BrowseAllPushButton:
					ChangeTab(AllTab);
					break;
				
				//case BrowseGoButton:
				//	RandomNewRecord();
				//	break;
					
				default:
					break;
			
			}
			break;
			
		case ctlRepeatEvent:
			switch(event->data.ctlSelect.controlID){
				
				case BrowseScrollUpRepeating:
					WordListBackward(false);
					break;
							
				case BrowseScrollDownRepeating:
					WordListForward(false);
					break;
			}
			break;
			
		case tblSelectEvent:
			TblUnhighlightSelection(event->data.tblSelect.pTable);
			CurrentRecord = TblGetRowID(event->data.tblSelect.pTable, event->data.tblSelect.row);
			
			switch(event->data.tblSelect.column){
				case statusColumn:
					FrmPopupForm(DetailForm);
					break;
					
				case entryColumn:
			
					TopVisibleFieldIndex = 0;
					EditRowIDWhichHadFocus = FirstFieldIndex;
					EditFieldPosition = 0;
			
					FrmGotoForm(EditForm);
					break;
				
				case arrowColumn:
					switch(tabCurrentSelected){
						case StudyListTab:
							MoveToRawList(event->data.tblSelect.row);
							break;
							
						case RawTab:
							MoveToStudyList(event->data.tblSelect.row);
							break;
							
						case AllTab:
							form = FrmInitForm(StatusForm);
							FrmDoDialog(form);
							FrmDeleteForm(form);
							break;
					}
					break;
				}
			handled = true;
			break;
			
		case penDownEvent:
			//capture pen down only if raw tab is displayed
			if (tabCurrentSelected == RawTab){
				DetectPenDown(event);
			}	
			break;
				
		case keyDownEvent:
		
			//if (TxtCharIsHardKey(event->data.keyDown.modifiers, event->data.keyDown.chr))
			//{
			//}

			if (EvtKeydownIsVirtual(event))
			{
				if (event->data.keyDown.chr == pageUpChr){
					//ctl = GetObjectPtr(BrowseScrollUpRepeating);
					//if (CtlGetValue(ctl) != 0)
					WordListBackward(false);
					
					if (!(event->data.keyDown.modifiers & autoRepeatKeyMask)) handled = true;
				}
				
				else if (event->data.keyDown.chr == pageDownChr){
					//ctl = GetObjectPtr(BrowseScrollDownRepeating);
					//if (CtlGetValue(ctl) != 0)
					WordListForward(false);
					
					if (!(event->data.keyDown.modifiers & autoRepeatKeyMask)) handled = true;
				}
			}
		

			else if (event->data.keyDown.chr == linefeedChr)
			{
				handled = true;
			}

			else {
				InstantLookup(event);
				handled = true;
			}
			
		break;
		
		case fldEnterEvent:
			FldHandleEvent(event->data.fldEnter.pField, event);
			
			p = FldGetTextPtr(event->data.fldEnter.pField);
			if ((p != NULL) && (StrLen(p) > 0))
			FldSetSelection(event->data.fldEnter.pField, 0, StrLen(p));
			
			if (tabCurrentSelected == RawTab){
				SearchFieldTicking = true;
				SearchFieldTicks = TimGetTicks();
			}
		
			handled = true;
			break;
		

		case fldChangedEvent:
			InstantLookup(event);
			handled = true;
			break;
		
		case nilEvent:
			x = TimGetTicks();
			if ((tabCurrentSelected == RawTab) &&
				SearchFieldTicking && (( x - SearchFieldTicks) > 500)){
				SearchFieldTicking = false;
				UpdateSearchField();
			}
			break;
			
		default:
			break;
	}
	
	return handled;
}

static void BrowseFormInit( FormType* frmP ){

	Int16 row;
	TableType* tblP;
	ControlPtr ctl;

	if (ShowAllCategories) CurrentCategory = dmAllCategories;
	
	// Initialize the word list table.
	tblP = GetObjectPtr(BrowseWordListTable);

	for (row = 0; row < RowsInTable; row++)
	{
		TblSetItemStyle(tblP, row, statusColumn, customTableItem);
		TblSetItemStyle(tblP, row, entryColumn, customTableItem);
		TblSetItemStyle(tblP, row, arrowColumn, customTableItem);
		TblSetRowUsable(tblP, row, false);
	}

	TblSetColumnUsable(tblP, statusColumn, true);
	TblSetColumnUsable(tblP, entryColumn, true);
	TblSetColumnUsable(tblP, arrowColumn, true);

	TblSetColumnSpacing(tblP, statusColumn, columnSpacing);
	
	// Set the callback routine that will draw the records.
	TblSetCustomDrawProcedure (tblP, statusColumn, WordListDrawRecord);
	TblSetCustomDrawProcedure (tblP, entryColumn, WordListDrawRecord);
	TblSetCustomDrawProcedure (tblP, arrowColumn, WordListDrawRecord);

	// Set the label of the category trigger.
	ctl = GetObjectPtr(BrowseCategoryPopTrigger);
	CategoryGetName (CurrentDB, CurrentCategory, CategoryName);
	CategorySetTriggerLabel (ctl, CategoryName);
	
	ChangeTab(tabCurrentSelected);
}

static void BrowseFormLoadTable( FormType* frmP )
{
	Int16      row = 0;
	TableType* 	tblP;
	UInt16 cursor = 0;
	Boolean full = false;
	Boolean reachedEnd = false;
	Boolean reachedHead = false;
	Int16 i;
	UInt16 studyNums; 
	UInt16 index = 0, recordIndex;
	HeaderItem item;
	Boolean detecting = true;
	UInt32 ticks, ticksPerSecond;
	Err error;

	tblP = GetObjectPtr(BrowseWordListTable);
	
	//	first branch: load study list
	if (tabCurrentSelected == StudyListTab){
				
		//Validation
		studyNums = GetHeaderItemNumber(CurrentDB, LearningRecordIndex);
		
		if (studyNums == 0){
			TopVisibleRecord = 0;
			index = 0;
			row = 0;
		}
		
		else {		
			if (TopVisibleRecord + 10 > studyNums - 1){
				if (studyNums < 11) TopVisibleRecord = 0;
				else TopVisibleRecord = studyNums - 11;
			} 
		
			if (studyNums > RowsInTable) VisibleRows =  RowsInTable;
			else VisibleRows = studyNums;
		
			for (index = TopVisibleRecord; index < TopVisibleRecord + VisibleRows; index ++){
				GetHeaderItem(CurrentDB, LearningRecordIndex, &item, index);
			
				error = DmFindRecordByID(CurrentDB, item, &recordIndex);
				if (error) ErrAlert(error);
			
				TblSetRowID(tblP, row, recordIndex);
				TblSetRowUsable(tblP, row, true);
				row ++;
			}
		}
		
		//	hide empty row
		while(row < RowsInTable){
			TblSetRowUsable(tblP, row, false);
			row ++;
		}
		
		//	draw table
		TblDrawTable(tblP);
		
		//	update search/count field
		UpdateSearchField();
		
		//	update scrollers				
		reachedHead = (TopVisibleRecord == 0);
		reachedEnd = ((TopVisibleRecord + RowsInTable) >= studyNums);
		
		FrmUpdateScrollers(frmP, FrmGetObjectIndex(frmP, BrowseScrollUpRepeating),
			FrmGetObjectIndex(frmP, BrowseScrollDownRepeating), !reachedHead, !reachedEnd);

		return;
	} 
	
	
	ticks = TimGetTicks();
	ticksPerSecond = (UInt32)SysTicksPerSecond();
	
	// 	For each row in the table, store the record number as the row id.
	//	first, check if table full
	if (VisibleRows == RowsInTable) full = true;
	
	//	forward search to fill table
	if (!full){

		if (VisibleRows == 0) cursor = TopVisibleRecord;
		else cursor = TblGetRowID(tblP, VisibleRows - 1) + 1;
		
		while (SeekNextRecord(&cursor, dmSeekForward)){
		
			if (detecting && ((TimGetTicks() - ticks) > ticksPerSecond / 2)){
				FntSetFont(largeBoldFont);
				WinDrawChars("Searching....", 9, 50, 70); 
				detecting = false;
			} 
		
			TblSetRowID(tblP, VisibleRows, cursor);
			TblSetRowUsable(tblP, VisibleRows, true);
		
			VisibleRows ++;
			cursor ++;
		
			if (VisibleRows == RowsInTable){
				full = true;
				break;
			}
		}
	}
	
	//	backward search to fill table	
	if (!full) {
		
		reachedEnd = true;
	
		if (VisibleRows == 0) cursor = TopVisibleRecord - 1;
		else cursor = TblGetRowID(tblP, 0) - 1;
	
		while (SeekNextRecord(&cursor, dmSeekBackward)){
		
			if (detecting && ((TimGetTicks() - ticks) > ticksPerSecond / 2)){
				FntSetFont(largeBoldFont);
				WinDrawChars("Searching....", 9, 50, 70); 
				detecting = false;
			} 

			for (i = VisibleRows; i > 0; i--)
				TblSetRowID(tblP, i, TblGetRowID(tblP, i-1));
				
			TblSetRowID(tblP, 0, cursor);
			TblSetRowUsable(tblP, VisibleRows, true);
						
			VisibleRows ++;
			cursor --;
		
			if (VisibleRows == RowsInTable){
				full = true;
				break;
			}
		}
		
		if (!full) reachedHead = true;
	}
	
	if (VisibleRows == 0){
		reachedHead = true;
		reachedEnd = true;
	}
	
	if (!reachedHead) {
		cursor = TblGetRowID(tblP, 0) - 1;
		reachedHead = !SeekNextRecord(&cursor, dmSeekBackward);
	}
	
	if (!reachedEnd) {
		cursor = TblGetRowID(tblP, VisibleRows - 1) + 1;	
		reachedEnd = !SeekNextRecord(&cursor, dmSeekForward);
	}

	// Hide the item that don't have any data.
	row = VisibleRows;
	while (row < RowsInTable)
	{
		TblSetRowUsable (tblP, row, false);
		row++;
	}
	
	TblDrawTable(tblP);
		
	//update scrollers
	FrmUpdateScrollers(frmP, FrmGetObjectIndex(frmP, BrowseScrollUpRepeating),
		FrmGetObjectIndex(frmP, BrowseScrollDownRepeating), !reachedHead, !reachedEnd);

	//update top visible record
	if (VisibleRows) TopVisibleRecord = TblGetRowID(tblP, 0);

	//	update search field if 
	//	2. raw tab and search field not focused
	if 	((tabCurrentSelected == RawTab) && (!SearchFieldTicking))
		UpdateSearchField();

}

static void	WordListDrawRecord (void * table, Int16 row, Int16 column, RectanglePtr bounds){

	UInt16 recordNum;
	Err error;
	WordRecordType record;
	MemHandle recordH;
	Int16 strWidth = 75;	//75 pixel at most for question field
	Int16 strLength = 0;
	Int16 offset = 0;
	Boolean fitWithinWidth = true;
	char* strP;
	char arrowChar;
	char infoChar;
	HeaderQueue queue;
	
	// Get the record number that corresponds to the table item to draw.
	// The record number is stored in the "intValue" field of the item.
	recordNum = TblGetRowID (table, row);

	error = GetWordRecord (CurrentDB, recordNum, &record, &recordH);
	if (error)
	{
		ErrNonFatalDisplay ("Record not found");
		return;
	}

	switch (column)
	{
		case entryColumn:
			if (record.field[WordFieldQuestion] != NULL) {
				strP = record.field[WordFieldQuestion];
				FntSetFont(boldFont);
				strLength = StrLen(strP);
				FntCharsInWidth(strP, &strWidth, &strLength, &fitWithinWidth);
				
				WinDrawChars(strP, strLength, bounds->topLeft.x, bounds->topLeft.y);
			}
			
			if (record.field[WordFieldAnswer] != NULL){
				strP = record.field[WordFieldAnswer];
				FntSetFont(stdFont);
				strLength = StrLen(strP);
				offset = strWidth + 10;
				strWidth = TblGetColumnWidth(table, entryColumn) - offset;
				FntCharsInWidth(strP, &strWidth, &strLength, &fitWithinWidth);
				
				WinDrawChars(strP, strLength, bounds->topLeft.x + 
					TblGetColumnWidth(table, entryColumn) - strWidth, bounds->topLeft.y);
			}
				
			break;
		case statusColumn:
			FntSetFont(symbolFont);
			infoChar = 10;
			WinDrawChars(&infoChar, 1, bounds->topLeft.x + 2, bounds->topLeft.y);
			FntSetFont(stdFont);
			break;
		
		case arrowColumn:
			
			switch(tabCurrentSelected){
				case StudyListTab:
					arrowChar = 187;
					break;
				case RawTab:
					arrowChar = 171;
					break;
				case AllTab:
					queue = GetQueue(CurrentDB, TblGetRowID(table, row));			
					arrowChar = WordQueueToChar(queue);
					break;
			}

			FntSetFont(boldFont);
			WinDrawChars(&arrowChar, 1, bounds->topLeft.x + 1, bounds->topLeft.y);
			FntSetFont(stdFont);
			
			break;
		
	}

	MemHandleUnlock(recordH);
}

UInt16 WordListSelectCategory (void)
{
	FormType* frmP;
	UInt16 category;
	Boolean categoryEdited;

	// Process the category popup list.
	category = CurrentCategory;

	frmP = FrmGetActiveForm();
	categoryEdited = CategorySelect (CurrentDB, frmP, BrowseCategoryPopTrigger,
			BrowseWordCategoryList, true, &category, CategoryName, 1, categoryDefaultEditCategoryString);

	if (category == dmAllCategories)
		ShowAllCategories = true;
	else
		ShowAllCategories = false;

	if ( categoryEdited || (category != CurrentCategory))
	{
		ToolsChangeCategory (category);

		// Display the new category.
		VisibleRows = 0;
		BrowseFormLoadTable(frmP);

		// By changing the category the current record is lost.
		CurrentRecord = noRecord;
	}

	return (category);
}

void ToolsChangeCategory (UInt16 category)
{
	CurrentCategory = category;
	TopVisibleRecord = FirstWordRecord;
}

void WordListForward(Boolean byline){
	
	FormType * frmP;
	TableType * tblP = GetObjectPtr(BrowseWordListTable);
	UInt16 steps;
	UInt16 cursor;
	Int16 i;
	Boolean reachedEnd = false;

	if (tabCurrentSelected == StudyListTab){
	
		if (TopVisibleRecord + RowsInTable >= GetHeaderItemNumber(CurrentDB, LearningRecordIndex)) return; 
			
		if (byline) TopVisibleRecord ++;
		else TopVisibleRecord += RowsInTable - 1;
	
		BrowseFormLoadTable(FrmGetActiveForm());
		
		return;
	}
	
	if (byline) steps = 1;
	else steps = RowsInTable - 1;
	
	cursor = TblGetRowID(tblP, VisibleRows - 1) + 1;
	
	while (steps > 0){
		if (!SeekNextRecord(&cursor, dmSeekForward)) {
			reachedEnd = true;
			break;
		}
		
		for (i = 0; i < VisibleRows - 1; i++)
			TblSetRowID(tblP, i, TblGetRowID(tblP, i + 1));
		
		TblSetRowID(tblP, VisibleRows - 1, cursor);
		
		cursor++;
		steps--;
	}
	
	if (TopVisibleRecord == TblGetRowID(tblP, 0)) return;
	
	TopVisibleRecord = TblGetRowID(tblP, 0);
		
	TblDrawTable(tblP);
		
	if (!reachedEnd) reachedEnd = !SeekNextRecord(&cursor, dmSeekForward);
	
	frmP = FrmGetActiveForm();
	FrmUpdateScrollers(frmP, FrmGetObjectIndex(frmP, BrowseScrollUpRepeating),
		FrmGetObjectIndex(frmP, BrowseScrollDownRepeating), true, !reachedEnd);
}

void WordListBackward(Boolean byline){
	FormType * frmP;
	TableType * tblP = GetObjectPtr(BrowseWordListTable);
	UInt16 steps;
	UInt16 cursor;
	Int16 i;
	
	Boolean reachedHead = false;

	if (tabCurrentSelected == StudyListTab){
		if (byline) steps = 1;
		else steps = 10;
		
		if (TopVisibleRecord == 0) return;
		
		while (steps > 0 && TopVisibleRecord > 0){
			TopVisibleRecord--;
			steps--;
		}
	
		BrowseFormLoadTable(FrmGetActiveForm());
		
		return;
	}

	if (byline) steps = 1;
	else steps = RowsInTable - 1;
	
	cursor = TblGetRowID(tblP, 0) - 1;
	
	while (steps > 0){
		if (!SeekNextRecord(&cursor, dmSeekBackward)) {
			reachedHead = true;
			break;
		}
		
		for (i = VisibleRows - 1; i > 0; i--)
			TblSetRowID(tblP, i, TblGetRowID(tblP, i - 1));

		TblSetRowID(tblP, 0, cursor);  
		cursor--;
		steps--;
	}	
	
	if (TopVisibleRecord == TblGetRowID(tblP, 0)) return;
	
	TopVisibleRecord = TblGetRowID(tblP, 0); 
	
	TblDrawTable(tblP);
	
	if (!reachedHead)
		reachedHead = !SeekNextRecord(&cursor, dmSeekBackward);
	
	frmP = FrmGetActiveForm();
	FrmUpdateScrollers(frmP, FrmGetObjectIndex(frmP, BrowseScrollUpRepeating), 
		FrmGetObjectIndex(frmP, BrowseScrollDownRepeating), !reachedHead, true);
	
}

//Boolean 
Boolean SeekNextRecord(UInt16 * indexP, Int16 direction){
	MemHandle recordH;
	HeaderRecord* headerP;
	Boolean found = false;
	
	if (direction == dmSeekForward){
	
		while(!found) {
			
			if (DmSeekRecordInCategory(CurrentDB, indexP, 0, direction, CurrentCategory)) return false;
			
			recordH = DmQueryRecord(CurrentDB, *indexP);
			headerP = MemHandleLock(recordH);
			if (headerP -> header.allBits & HeaderMask.allBits) found = true;
			else {
				found = false;
				(*indexP)++;
			}
			
			MemHandleUnlock(recordH);
		
		} 
		return found;
	}
	
	if (direction == dmSeekBackward){
	
		while (!found){
			
			if (DmSeekRecordInCategory(CurrentDB, indexP, 0, direction, CurrentCategory)) return false;
			
			if ((*indexP)<2) return false;
			
			recordH = DmQueryRecord(CurrentDB, *indexP);
			headerP = MemHandleLock(recordH);
			
			if (headerP -> header.allBits & HeaderMask.allBits) found = true;
			else {
				found = false;
				(*indexP)--;
			}
		
			MemHandleUnlock(recordH);
	
		}
		return found;
	}
	
	return found;
}

void ChangeTab(TabSelected tab){

	FormType* frmP;
	TableType* tblP;
	ControlType* ctlP;

	frmP = FrmGetActiveForm();
	tblP = GetObjectPtr(BrowseWordListTable);
	ctlP = GetObjectPtr(BrowseNewButton);
	
	//
	if (FrmGetControlGroupSelection(frmP, 1) == frmNoSelectedControl)
		FrmSetControlGroupSelection(frmP, 1, BrowseWordListTable + 1 + tab);

	//save top visible record in current tab
	TopVisibleBackup[tabCurrentSelected] = TopVisibleRecord;
	
	//change to new tab
	tabCurrentSelected = tab;
	
	//Load top visible record in backup
	TopVisibleRecord = TopVisibleBackup[tabCurrentSelected];
	VisibleRows = 0;
	
	//prepare headermask
	switch(tabCurrentSelected){
		case RawTab:
			HeaderMask.allBits = 0;
			SetBitMacro(HeaderMask.allBits, RawQueue);
			break;
			
		case StudyListTab:
			HeaderMask.allBits = 0;
			SetBitMacro(HeaderMask.allBits, LearningQueue);
			break;
			
		case AllTab:
			HeaderMask.allBits = 0;
			SetBitMacro(HeaderMask.allBits, RawQueue);
 			SetBitMacro(HeaderMask.allBits, LearningQueue);
 			SetBitMacro(HeaderMask.allBits, TestingQueue);
 			SetBitMacro(HeaderMask.allBits, WaitingQueue);
 			break;
	}
	
	//change display
	switch(tabCurrentSelected){
		case StudyListTab:
			HideObject(BrowseSearchField);
			HideObject(BrowseCategoryPopTrigger);
			ShowObject(BrowseRemoveAllButton);
			ShowObject(BrowseShuffleButton);
			HideObject(BrowseAddAllButton);
			HideObject(BrowseNewButton);										
			break;
		
		case RawTab:
			HideObject(BrowseSearchField);
			ShowObject(BrowseCategoryPopTrigger);
			HideObject(BrowseRemoveAllButton);
			HideObject(BrowseShuffleButton);
			ShowObject(BrowseAddAllButton);
			HideObject(BrowseNewButton);			
			break;
		
		case AllTab:
			ShowObject(BrowseSearchField);
			ShowObject(BrowseCategoryPopTrigger);		
			HideObject(BrowseRemoveAllButton);
			HideObject(BrowseShuffleButton);
			HideObject(BrowseAddAllButton);
			ShowObject(BrowseNewButton);			
			break;
	}
	
	SearchFieldTicking = false;

	ClearWordListTable();
	FrmDrawForm(frmP);
	DrawTitleBar();
	
	BrowseFormLoadTable(frmP);
}

void DrawTitleBar(){
	WinPushDrawState();
	WinSetForeColor(UIColorGetTableEntryIndex(UIObjectSelectedFill));
	WinDrawLine(0, 13, 160, 13);
	WinDrawLine(0, 14, 160, 14);
	WinPopDrawState();

	//WinErasePixel(0, 0);
	//WinErasePixel(53, 0);
	//WinErasePixel(80, 0);
	//WinErasePixel(97, 0);
}

void MoveToStudyList(Int16 row){
	
	TableType * table = GetObjectPtr(BrowseWordListTable);
	UInt16 index = TblGetRowID(table, row);

	SetQueue(CurrentDB, index, LearningQueue);
	
	//first pack raw list item
	while (row < VisibleRows - 1){
		TblSetRowID(table, row, TblGetRowID(table, row + 1));
		row ++;
	}
	
	VisibleRows--;
	if (VisibleRows > 0) TopVisibleRecord = TblGetRowID(table, 0);
	
	BrowseFormLoadTable(FrmGetActiveForm());
		
}

void BatchMoveToStudyList(){

	TableType * table = GetObjectPtr(BrowseWordListTable);
	Int16 row;
	UInt16 index;
	
	if (VisibleRows == 0) return;
		
	for (row = 0; row < VisibleRows; row ++){

		index = TblGetRowID(table, row);
		SetQueue(CurrentDB, index, LearningQueue);
		
	}
	
	VisibleRows = 0;
		
	BrowseFormLoadTable(FrmGetActiveForm());
}

void MoveToRawList(Int16 row){

	TableType * table = GetObjectPtr(BrowseWordListTable);
	UInt16 index = TblGetRowID(table, row);
	SetQueue(CurrentDB, index, RawQueue);
	
	BrowseFormLoadTable(FrmGetActiveForm());
}

void BatchMoveToRawList(){

	TableType * table = GetObjectPtr(BrowseWordListTable);
	Int16 row;
	UInt16 index;
		
	if (VisibleRows == 0) return;
	
	for (row = 0; row < VisibleRows; row++){
	
		index = TblGetRowID(table, row);
		SetQueue(CurrentDB, index, RawQueue);	
	}
	
	VisibleRows = 0;
	
	BrowseFormLoadTable(FrmGetActiveForm());
}

void UpdateSearchField(){
	
	FormType* frmP;
	RectangleType bounds;
	FontID currFont, studylistFont, rawFont;
	UInt16 rawNumbers, studylistNumbers;
	Char rawStr[maxStrIToALen], studylistStr[maxStrIToALen];
	Char slash[4];
	
	UInt16 charsWidth = 0;	
	
	if ((tabCurrentSelected == RawTab) && SearchFieldTicking) return;
	
	StrPrintF(slash, "%s", " / ");
	
	rawNumbers = GetRawWordCount();
	studylistNumbers = GetHeaderItemNumber(CurrentDB, LearningRecordIndex);

	StrIToA(rawStr, rawNumbers);
	StrIToA(studylistStr, studylistNumbers);
	
	frmP = FrmGetActiveForm();
	FrmHideObject(frmP, FrmGetObjectIndex(frmP, BrowseSearchField));
	
	studylistFont = stdFont;
	rawFont = stdFont;
	if (tabCurrentSelected == StudyListTab) studylistFont = boldFont;
	if (tabCurrentSelected == RawTab) rawFont = boldFont;
	
	currFont = FntSetFont(studylistFont);
	charsWidth += FntCharsWidth(studylistStr, StrLen(studylistStr));
	
	FntSetFont(stdFont);
	charsWidth += FntCharsWidth(slash, StrLen(slash));
	
	FntSetFont(rawFont);
	charsWidth += FntCharsWidth(rawStr, StrLen(rawStr));
	
	FrmGetObjectBounds(frmP, FrmGetObjectIndex(frmP, BrowseSearchField), &bounds);
	
	if (bounds.extent.x > charsWidth)
		charsWidth = (bounds.extent.x - charsWidth) / 2;
	else 
		charsWidth = 0;
	
	WinEraseRectangle(&bounds, 0);
	
	FntSetFont(studylistFont);
	WinDrawChars(studylistStr, StrLen(studylistStr), bounds.topLeft.x + charsWidth, 
		bounds.topLeft.y + bounds.extent.y - FntLineHeight());
	charsWidth += FntCharsWidth(studylistStr, StrLen(studylistStr));
	
	FntSetFont(stdFont);
	WinDrawChars(slash, StrLen(slash), bounds.topLeft.x + charsWidth, 
		bounds.topLeft.y + bounds.extent.y - FntLineHeight());
	charsWidth += FntCharsWidth(slash, StrLen(slash));
	
	FntSetFont(rawFont);
	WinDrawChars(rawStr, StrLen(rawStr), bounds.topLeft.x + charsWidth, 
		bounds.topLeft.y + bounds.extent.y - FntLineHeight());
	
	FntSetFont(currFont);
}

/*	
void RandomNewRecord ()
{
	WordRecordType newRecord;
	WordFields i;
	UInt16 j;
	UInt16 attr;
	Err err;
	Char a, b, c, d;
	Char* pq;
	Char* pa;
	MemHandle questionH;
	MemHandle answerH;
	
	MemorizingStatus memoStatus ={2.50, //e-factor
		40, //difficulty 
		0, //total lapse
		0, //total recall
		0, //recent lapse
		0, //recent recall
		0, //step
		0, //last test
		0, //current test
		0,	//next test
		0,	//first learn
		0}; //create

	DateType today;
	UInt32 seconds;
	UInt32 seed;
	
	newRecord.header.allBits = 0;
	SetBitMacro(newRecord.header.allBits, RawQueue);
	
	newRecord.ltStatus.learningStatus = 0;
	newRecord.memoStatus = memoStatus;
	
	seconds = TimGetSeconds();
	seed = seconds;
	DateSecondsToDate(seconds, &today);
	newRecord.memoStatus.create = today;
	
	for (j = 0; j < 1000; j++){
		questionH = MemHandleNew(10);
		answerH = MemHandleNew (10);
	
		seed = seed * 241;
		a = seed % 26 + 65;
		seed = seed * 241;
		b = seed % 26 + 65;
		seed = seed * 241;
		c = seed % 26 + 65;
		seed = seed * 241;
		d = seed % 26 + 65;
	
		pq = MemHandleLock(questionH);
		StrPrintF(pq, "%c%c%c%c%c", a, b, c, d, '\0');
	
		pa = MemHandleLock(answerH);
		StrPrintF(pa, "%c%c%c%c%c", d, c, b, a, '\0');
	
		for (i = WordFieldQuestion; i < WordFieldsCount; i++)
			newRecord.field[i] = NULL;
	
		newRecord.field[WordFieldQuestion] = pq;
		newRecord.field[WordFieldAnswer] = pa;

		err = NewWordRecord(CurrentDB, &newRecord, &CurrentRecord);
		if (err)
		{
			FrmAlert(DeviceFullAlert);
			return;
		}

		// Set it's category to the category being viewed.
		// If the category is All then set the category to unfiled.
		DmRecordInfo (CurrentDB, CurrentRecord, &attr, NULL, NULL);
		attr &= ~dmRecAttrCategoryMask;
		attr |= ((CurrentCategory == dmAllCategories) ? dmUnfiledCategory :
			 CurrentCategory) | dmRecAttrDirty;
		DmSetRecordInfo (CurrentDB, CurrentRecord, &attr, NULL);
		
		IncrementRawWordCount();
				
		MemHandleUnlock(questionH);
		MemHandleUnlock(answerH);
		MemHandleFree(questionH);
		MemHandleFree(answerH);
		
	}
	// Set the global variable that determines which field is the top visible
	// field in the edit view.  Also done when New is pressed.
	TopVisibleRecord = CurrentRecord;
	VisibleRows = 0;
	
	BrowseFormLoadTable(FrmGetActiveForm());
	FrmDrawForm(FrmGetActiveForm());
}
*/

void ClearWordListTable(){
	
	TableType * tblP = GetObjectPtr(BrowseWordListTable);
	int i;
	
	for (i = 0; i < RowsInTable; i ++)
		TblSetRowUsable(tblP, i, false);

}

void InstantLookup(EventType * event){
	
	FormType * frmP = FrmGetActiveForm();
	FieldType * fldP;
	Char * fldTextP;
	UInt16 fldIndex;

	fldP = GetObjectPtr (BrowseSearchField);

	if ((tabCurrentSelected == RawTab) && !SearchFieldTicking) return;
	
	fldIndex = GetObjectIndex(BrowseSearchField);
	fldTextP = FldGetTextPtr(fldP);
	
	FrmSetFocus(frmP, fldIndex);

	if (FldHandleEvent (fldP, event) || event->eType == fldChangedEvent)
	{
		fldTextP = FldGetTextPtr(fldP);
		if (fldTextP != NULL && StrLen(fldTextP) > 0){
			TopVisibleRecord = InstantLookupPosition(CurrentDB, fldTextP);
			VisibleRows = 0;
			BrowseFormLoadTable(frmP);
		}
	}
	
	if (tabCurrentSelected == RawTab) SearchFieldIdle(fldP);
}	

void SearchFieldIdle(FieldType* fldP){

	SearchFieldTicking = true;
	SearchFieldTicks = TimGetTicks();	
		
}

//void	NumMoveToStudyList(){
//	FormType * form;
//	
//	form = FrmInitForm(NumRawForm);
//	FrmDoDialog(form);
//	FrmDeleteForm(form);
//}


void ShuffleStudyList(){

	if (DoCustomDialog("Confirm", "This operation will shuffle the study list.", false, NULL, NULL, true)){
	
		ShuffleHeaderItem(CurrentDB, LearningRecordIndex);
		BrowseFormLoadTable(FrmGetActiveForm());
	}

}

void DetectPenDown(EventType* event){
	
	FormType* frmP;
	RectangleType bounds;
	FieldType* field;
	
	frmP = FrmGetActiveForm();
	FrmGetObjectBounds(frmP, FrmGetObjectIndex(frmP, BrowseSearchField), &bounds);
	
	if (RctPtInRectangle(event->screenX, event->screenY, &bounds)){
		FrmShowObject(frmP, FrmGetObjectIndex(frmP, BrowseSearchField));
		
		field = GetObjectPtr(BrowseSearchField);
		
		FldDrawField(field);
	}
	else {
		SearchFieldTicking = false;
	//	UpdateSearchField();
	}
}

Char WordQueueToChar(HeaderQueue queue){
	
	switch (queue){
		case RawQueue:
			return 'R';
		
		case LearningQueue:
			return 'L';
		
		case TestingQueue:
			return 'T';
			
		case WaitingQueue:
			return 'W';
		
		case TestingQFlag:
		case LearningQFlag:
			return 'H';
			
		default:
			break;
	}
	
	return ' ';
}






