#include <PalmOS.h>
#include <PalmUtils.h>

#include "SugarMemo.h"
#include "SugarMemoRsc.h"
#include "SugarMemoDB.h"
#include "SugarMemoBrowse.h"
#include "SugarMemoEdit.h"
#include "SugarMemoTools.h"

#define labelColumnWidth 15
#define LabelLength 10

//typedef char EditLabel[LabelLength];

#define noFieldIndex		0xff

#define labelColumn 	0
#define dataColumn 		1

#define spaceAfterColumn	2

#define FirstFieldIndex	0
#define MaxFieldIndex  	3

#define ColumnsInVowelTable 	6
#define ColumnsInConsonantTable	2

//global
Char 	EditCategoryName[dmCategoryLength];
UInt16	TopVisibleFieldIndex =0;
UInt16	EditRowIDWhichHadFocus =0;
UInt16	EditFieldPosition = 0;

UInt16 	StudyListIndex;

extern TabSelected tabCurrentSelected;
extern Boolean 	SeekNextRecord(UInt16 * indexP, Int16 direction);


//internal
static 	UInt16 CurrentFieldIndex = 0;

const WordFields FieldMap[4] = {WordFieldQuestion, 
										WordFieldPhonetic, 
										WordFieldAnswer, 
										WordFieldNote3};


EditLabel fieldLabel[WordFieldsCount] = {"Q", 
												"P", 
												"A", 
												"WordNet", 
												"MerriamW",	
												"Dict1", 
												"Dict2", 
												"Dict3", 
												"Note1", 
												"Note2", 
												"N"};


//internal functions
static void EditInit( FormPtr frmP, Boolean leaveDataLocked );
static void EditFormLoadControls();
static void EditFormLoadCategory();
static void EditFormLoadTable(FormType* frmP);
static void EditInitTableRow( FormType* frmP, TablePtr table, UInt16 row, UInt16 fieldIndex, 
				Int16 rowHeight, WordRecordType* record);
static UInt16 EditGetFieldHeight (TablePtr table, UInt16 fieldIndex, Int16 columnWidth, Int16 maxHeight, 
	WordRecordType* record);
				
static Err	EditGetRecordField (void * table, Int16 row, Int16 column, Boolean editing, 
				MemHandle * textH, Int16 * textOffset, Int16 * textAllocSize, FieldPtr fld);
static Boolean	EditSaveRecordField (void * table, Int16 row, Int16 column);
static UInt16 EditFormSaveRecord ();
static void	EditDrawRecordField (void * table, Int16 row, Int16 column, RectanglePtr bounds);

static void EditUpdateScrollers (FormPtr frmP, UInt16 bottomFieldIndex, Boolean lastItemClipped);
static void EditFormScroll (WinDirectionType direction);
static void EditFormResizeField (EventType * event);
static UInt16 EditCategorySelection (void);
static void FieldSelection(Int16 row);
static void EditFormNextWord(Boolean next);
static void EditUpdateHorizonScrollers();
//event handler
Boolean EditFormHandleEvent(EventType * event){

	FormType* frmP = FrmGetActiveForm();
	Boolean handled = false;
	TableType* tableP = GetObjectPtr(EditEditTable);
	FieldType * field;
	ControlType * ctl = GetObjectPtr(EditLearnPushButton);
	
	switch (event->eType){
		
		case frmOpenEvent:
			frmP = FrmGetActiveForm ();
			EditInit (frmP, false);
			FrmDrawForm (frmP);
			handled = true;
			break;
		
		case frmUpdateEvent:
			FrmDrawForm(frmP);
			handled = true;
			break;
			
		case fldHeightChangedEvent:
			EditFormResizeField(event);
			handled = true;
			break;		
		
		case tblSelectEvent:
			FieldSelection(event->data.tblSelect.row);
			break;
		
		case ctlSelectEvent:
			switch (event->data.ctlSelect.controlID){
				case EditCategoryPopTrigger:
					EditCategorySelection();
					handled = true;
					break;
					
				case EditDoneButton:
					switch(HeaderRecordIndex){
						case HeaderIndexCounts:
							VisibleRows = 0;
							FrmGotoForm(BrowseForm);
							break;
						
						case LearningRecordIndex:
						case TestingRecordIndex:
							FrmGotoForm(Learn2Form);
							break;
					}
					
					handled = true;
					break;
					
				case EditDeleteButton:
					
					FrmSetFocus(frmP, noFocus);
					WordDBDeleteRecord(false);
					VisibleRows = 0;
					FrmGotoForm(BrowseForm);
					handled = true;
					break;
					
				case EditLearnPushButton:
					if (CtlGetValue(ctl) == 0)	SetQueue(CurrentDB, CurrentRecord, RawQueue);
					else	SetQueue(CurrentDB, CurrentRecord, LearningQueue);
					//do not set handled, let system update display
					break;
									
				default:
					break;
			
			}
			break;
			
		case ctlRepeatEvent:
			switch (event->data.ctlRepeat.controlID)
			{
				case EditScrollUpRepeating:
					EditFormScroll (winUp);
					// leave unhandled so the buttons can repeat
					break;
		
				case EditScrollDownRepeating:
					EditFormScroll (winDown);
					// leave unhandled so the buttons can repeat
					break;
					
				case EditPrevRepeating:
					EditFormNextWord(false);
					break;
					
				case EditNextRepeating:
					EditFormNextWord(true);
					break;
					
				default:
					break;
			}
			break;

		case keyDownEvent:
		
			if (EvtKeydownIsVirtual(event))
			{
				switch (event->data.keyDown.chr)
				{
					case vchrPageUp:
						if (TblGetLastUsableRow(tableP) == 0){
							
							field = TblGetCurrentField(tableP);
				
							if (field != NULL){
								
								if (FldScrollable(field, winUp)){
									EvtEnqueueKey(chrUpArrow, 0, 0);
									FldScrollField(field, 1, winUp);
									handled = true;
									break;
								}
							}
						}
						EditFormScroll (winUp);
						handled = true;
						break;
		
					case vchrPageDown:
						if (TblGetLastUsableRow(tableP) == 0){
							
							field = TblGetCurrentField(tableP);
				
							if (field != NULL){
								
								if (FldScrollable(field, winDown)){
									EvtEnqueueKey(chrDownArrow, 0, 0);
									FldScrollField(field, 1, winDown);
									handled = true;
									break;
								}
							}
						}
						
						EditFormScroll (winDown);
						handled = true;
						break;
				}
			}
			break;
		
		default:
			break;
	}
	
	return handled;
}

void EditFormLoadControls(){
	
	ControlType * ctl;
	HeaderQueue queue;

	//Initialize controls
	if (HeaderRecordIndex == HeaderIndexCounts){
		ShowObject(EditDeleteButton);
		ShowObject(EditPrevRepeating);
		ShowObject(EditNextRepeating);

		if (tabCurrentSelected == StudyListTab)
			HideObject(EditLearnPushButton);
		else {

			queue = GetQueue(CurrentDB, CurrentRecord);

			if (queue == RawQueue || queue == LearningQueue){
				ctl = GetObjectPtr(EditLearnPushButton);
		
				if (queue == RawQueue) CtlSetValue(ctl, 0);
				else CtlSetValue(ctl, 1);

				ShowObject(EditLearnPushButton);
		
			}
			else
			HideObject(EditLearnPushButton);	
		}
		
	}
	else { 
		HideObject(EditDeleteButton);
		HideObject(EditLearnPushButton);
		HideObject(EditPrevRepeating);
		HideObject(EditNextRepeating);
	}

}

void EditFormLoadCategory(){

	UInt16 attr;
	UInt16 category;
	ControlType* ctl;
	
	// Set the label of the category trigger.
	DmRecordInfo (CurrentDB, CurrentRecord, &attr, NULL, NULL);
	category = attr & dmRecAttrCategoryMask;

	ctl = GetObjectPtr(EditCategoryPopTrigger);
	CategoryGetName (CurrentDB, category, EditCategoryName);
	CategorySetTriggerLabel (ctl, EditCategoryName);
	
	CtlDrawControl(ctl);

}

//This routine initialize edit form
void EditInit( FormPtr frmP, Boolean leaveDataLocked ){
	UInt16 row;
	UInt16 rowsInTable;
	UInt16 dataColumnWidth;
	TablePtr table;
	RectangleType bounds;
	
	
	EditFormLoadControls();
	EditFormLoadCategory();
	if (HeaderRecordIndex == HeaderIndexCounts) EditUpdateHorizonScrollers();

		
	// Initialize the edit table.
	table = GetObjectPtr(EditEditTable);
	rowsInTable = TblGetNumberOfRows (table);
	
	for (row = 0; row < rowsInTable; row++)
	{
		// This sets the data column
		TblSetItemStyle (table, row, labelColumn, labelTableItem);
		TblSetRowUsable (table, row, false);
	}

	TblSetColumnUsable (table, labelColumn, true);
	TblSetColumnUsable (table, dataColumn, true);
	TblSetColumnSpacing (table, labelColumn, spaceAfterColumn);

	// Set the callback routines
	TblSetLoadDataProcedure (table, dataColumn, EditGetRecordField);
	TblSetSaveDataProcedure (table, dataColumn, EditSaveRecordField);
	TblSetCustomDrawProcedure (table, dataColumn, EditDrawRecordField);

	// Set the column widths so that the label column contents fit exactly.
	TblGetBounds (table, &bounds);
	dataColumnWidth = bounds.extent.x - spaceAfterColumn - labelColumnWidth; 

	TblSetColumnWidth(table, labelColumn, labelColumnWidth);
	TblSetColumnWidth(table, dataColumn, dataColumnWidth);

	EditFormLoadTable(frmP);
		
}

Err	EditGetRecordField (void * table, Int16 row, Int16 column, 
	Boolean editing, MemHandle * textH, Int16 * textOffset, Int16 * textAllocSize, FieldPtr fld){

	UInt16 fieldNum;
	UInt16  fieldIndex;
	Char * recordP;
	Char * fieldP;
	MemHandle recordH, fieldH;
	UInt16 fieldSize;
	WordRecordType record;
	//FieldType * field;

	FldSetMaxChars(fld, 1024);

	// Get the field number that corresponds to the table item.
	// The field number is stored as the row id.
	fieldIndex = TblGetRowID (table, row);
	fieldNum = FieldMap[fieldIndex];

	GetWordRecord (CurrentDB, CurrentRecord, &record, &recordH);

	if (editing)
	{

		if (record.field[fieldNum])
		{
			fieldSize = StrLen(record.field[fieldNum]) + 1;
			fieldH = MemHandleNew(fieldSize);	// Handle freeing done into PrvEditSaveRecordField() by calling FldFreeMemory() function
			fieldP = MemHandleLock(fieldH);
			MemMove(fieldP, record.field[fieldNum], fieldSize);
			*textAllocSize = fieldSize;
			MemHandleUnlock(fieldH);
		}
		else
		{
			fieldH = 0;
			*textAllocSize = 0;
		}
		MemHandleUnlock (recordH);
		*textOffset = 0;         // only one string
		*textH = fieldH;
		return (0);

	}
	else
	{
		// Calculate the offset from the start of the record.
		recordP = MemHandleLock (recordH);   // record now locked twice

		if (record.field[fieldNum])
		{
			*textOffset = record.field[fieldNum] - recordP;
			*textAllocSize = StrLen (record.field[fieldNum]) + 1;  // one for null terminator
		}
		else
		{
			do
			{
				fieldNum++;
			} while (fieldNum < WordFieldsCount && record.field[fieldNum] == NULL);

			if (fieldNum < WordFieldsCount)
				*textOffset = record.field[fieldNum] - recordP;
			else
				// Place the new field at the end of the text.
				*textOffset = MemHandleSize(recordH);

			*textAllocSize = 0;  // one for null terminator
		}
		
		MemHandleUnlock (recordH);   // unlock the second lock
	}

	MemHandleUnlock (recordH);      // unlock the AddrGetRecord lock

	*textH = recordH;
	return (0);
}

Boolean	EditSaveRecordField (void * table, Int16 row, Int16 column){
	UInt16 fieldNum;
	UInt16 fieldIndex;
	FieldPtr fld;
	WordRecordType record;
	MemHandle recordH;
	MemHandle textH;
	Char * textP;
	WordFieldFlags bit;
	Err err;
	Boolean redraw = false;
	UInt16 numOfRows;
	Int16 newSize;

	fld = TblGetCurrentField (table);
	textH = FldGetTextHandle(fld);

	// Get the field number that corresponds to the table item to save.
	fieldIndex = TblGetRowID (table, row);
	fieldNum = FieldMap[fieldIndex];

	// Save the field last edited.
	EditRowIDWhichHadFocus = fieldIndex;

	// Save the cursor position of the field last edited.
	// Check if the top of the text is scroll off the top of the
	// field, if it is then redraw the field.
	if (FldGetScrollPosition (fld))
	{
		FldSetScrollPosition (fld, 0);
		EditFieldPosition = 0;
	}
	else
		EditFieldPosition = FldGetInsPtPosition (fld);

	// Make sure there any selection is removed since we will free
	// the text memory before the callee can remove the selection.
	FldSetSelection (fld, 0, 0);


	if (FldDirty (fld))
	{
		// Since the field is dirty, mark the record as dirty
		ToolsDirtyRecord (CurrentRecord);

		// Get a pointer to the text of the field.
		if (textH == 0)
			textP = NULL;
		else
		{
			textP = MemHandleLock(textH);
			if (textP[0] == '\0')
				textP = NULL;
		}


		GetWordRecord (CurrentDB, CurrentRecord, &record, &recordH);
		record.field[fieldNum] = textP;


		bit.allBits = (UInt32)1 << fieldNum;
		err = ChangeWordRecord(CurrentDB, &CurrentRecord, &record, bit);

		// The new field has been copied into the new record.  Unlock it.
		if (textP)
			MemPtrUnlock(textP);

		// The change was not made (probably storage out of memory)
		if (err)
		{
			// Because the storage is full the text in the text field differs
			// from the text in the record.  PrvEditGetFieldHeight uses
			// the text in the field (because it's being edited).
			// Make the text in the field the same as the text in the record.
			// Resizing should always be possible.
			MemHandleUnlock(recordH);      // Get original text
			GetWordRecord(CurrentDB, CurrentRecord, &record, &recordH);

			if (record.field[fieldNum] == NULL)
				newSize = 1;
			else
				newSize = StrLen(record.field[fieldNum]) + 1;

			// Have the field stop using the chunk to unlock it.  Otherwise the resize can't
			// move the chunk if more space is needed and no adjacent free space exists.
			FldSetTextHandle (fld, 0);
			if (!MemHandleResize(textH, newSize))
			{
				textP = MemHandleLock(textH);
				if (newSize > 1)
					StrCopy(textP, record.field[fieldNum]);
				else
					textP[0] = '\0';
				MemPtrUnlock(textP);
			}
			else
			{
				ErrNonFatalDisplay("Resize failed.");
			}


			// Update the text field to use whatever text we have.
			FldSetTextHandle (fld, textH);

			MemHandleUnlock(recordH);
			FrmAlert(DeviceFullAlert);

			// The field may no longer be the same height.  This row and those
			// below may need to be recalced. Mark this row and those
			// below it not usable and reload the table.
			numOfRows = TblGetNumberOfRows(table);
			while (row < numOfRows)
			{
				TblSetRowUsable(table, row, false);
				row++;
			}
			EditFormLoadTable(FrmGetActiveForm());
			redraw = true;                  // redraw the table showing change lost
		}

	}

	// Free the memory used for the field's text because the table suppresses it.
	FldFreeMemory (fld);

	return redraw;
}

static void EditFormLoadTable( FormType* frmP )
{
	UInt16 row;
	UInt16 numRows;
	UInt16 lineHeight;
	UInt16 fieldIndex;
	UInt16 lastFieldIndex;
	UInt16 dataHeight;
	UInt16 tableHeight;
	UInt16 columnWidth;
	UInt16 pos, oldPos;
	UInt16 height, oldHeight;
	TablePtr table;
	Boolean rowUsable;
	Boolean rowsInserted = false;
	Boolean lastItemClipped;
	RectangleType r;
	WordRecordType record;
	MemHandle recordH;

	// Get the current record
	GetWordRecord (CurrentDB, CurrentRecord, &record, &recordH);

	// Get the height of the table and the width of edit column.
	table = FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, EditEditTable));
	TblGetBounds (table, &r);
	tableHeight = r.extent.y;
	columnWidth = TblGetColumnWidth (table, dataColumn);	
	
	// If we currently have a selected record, make sure that it is not
	// above the first visible record.
	if (CurrentFieldIndex != noFieldIndex)
	{
		if (CurrentFieldIndex < TopVisibleFieldIndex)
			TopVisibleFieldIndex = CurrentFieldIndex;
	}

	row = 0;
	dataHeight = 0;
	oldPos = pos = 0;
	fieldIndex = TopVisibleFieldIndex;
	lastFieldIndex = fieldIndex;
		
	// Load records into the table.
	while (fieldIndex <= MaxFieldIndex)
	{
		// Compute the height of the field's text string.
		height = EditGetFieldHeight (table, fieldIndex, columnWidth, tableHeight, &record);

		// Is there enought room for at least one line of the the decription.
		lineHeight = FntLineHeight ();

		if (tableHeight >= dataHeight + lineHeight)
		{
			rowUsable = TblRowUsable (table, row);

			// Get the height of the current row.
			if (rowUsable)
				oldHeight = TblGetRowHeight (table, row);
			else
				oldHeight = 0;

			// If the field is not already being displayed in the current
			// row, load the field into the table.
			if ((! rowUsable) || (TblGetRowID (table, row) != fieldIndex))
				EditInitTableRow(frmP, table, row, fieldIndex, height, &record);


			// If the height or the position of the item has changed draw the item.
			else if (height != oldHeight)
			{
				TblSetRowHeight (table, row, height);
				TblMarkRowInvalid (table, row);
			}
			else if (pos != oldPos)
			{
				TblMarkRowInvalid (table, row);
			}

			pos += height;
			oldPos += oldHeight;
			lastFieldIndex = fieldIndex;
			fieldIndex++;
			row++;
		}
		
		dataHeight += height;


		// Is the table full?
		if (dataHeight >= tableHeight)
		{
			// If we have a currently selected field, make sure that it is
			// not below the last visible field.  If the currently selected
			// field is the last visible record, make sure the whole field
			// is visible.
			if (CurrentFieldIndex == noFieldIndex)
				break;

			// Above last visible?
			else if  (CurrentFieldIndex < fieldIndex)
				break;

			// Last visible?
			else if (fieldIndex == lastFieldIndex)
			{
				if ((fieldIndex == TopVisibleFieldIndex) || (dataHeight == tableHeight))
					break;
			}

			// Remove the top item from the table and reload the table again.
			TopVisibleFieldIndex++;
			fieldIndex = TopVisibleFieldIndex;


			// Below last visible.
			//			else
			//				TopVisibleFieldIndex = CurrentFieldIndex;

			row = 0;
			dataHeight = 0;
			oldPos = pos = 0;
		}
	}

	
	// Hide the item that don't have any data.
	numRows = TblGetNumberOfRows (table);
	while (row < numRows)
	{
		TblSetRowUsable (table, row, false);
		row++;
	}

	// If the table is not full and the first visible field is
	// not the first field	in the record, displays enough fields
	// to fill out the table by adding fields to the top of the table.
	while (dataHeight < tableHeight)
	{
		fieldIndex = TopVisibleFieldIndex;
		if (fieldIndex == 0) break;
		fieldIndex--;

		// Compute the height of the field.
		height = EditGetFieldHeight (table, fieldIndex,	columnWidth, tableHeight, &record);


		// If adding the item to the table will overflow the height of
		// the table, don't add the item.
		if (dataHeight + height > tableHeight)
			break;

		// Insert a row before the first row.
		TblInsertRow (table, 0);

		EditInitTableRow(frmP, table, 0, fieldIndex, height, &record);

		TopVisibleFieldIndex = fieldIndex;

		rowsInserted = true;

		dataHeight += height;
	}
	
	// If rows were inserted to full out the page, invalidate the whole
	// table, it all needs to be redrawn.
	if (rowsInserted)
		TblMarkTableInvalid (table);

	// If the height of the data in the table is greater than the height
	// of the table, then the bottom of the last row is clip and the
	// table is scrollable.
	lastItemClipped = (dataHeight > tableHeight);

	// Update the scroll arrows.
	EditUpdateScrollers(frmP, lastFieldIndex, lastItemClipped);

	MemHandleUnlock(recordH);
	
	TblDrawTable(table);
}

//Get the field height
UInt16 EditGetFieldHeight (TablePtr table, UInt16 fieldIndex, Int16 columnWidth, Int16 maxHeight, 
	WordRecordType* record)
{
	Int16 row;
	Int16 column;
	UInt16 index;
	Int16 height = 0;
	UInt16 lineHeight;
	Char * str;
	FieldPtr fld;
	FontID currFont;
	
	if (FieldMap[fieldIndex] == WordFieldPhonetic) {
		currFont = FntSetFont(PhoneticSmallFont);
		height = 0;
		height += FntLineHeight();
		FntSetFont(currFont);
		return (height);
	}
	
	if (TblEditing (table))
	{
		TblGetSelection (table, &row, &column);
		if (fieldIndex == TblGetRowID (table, row))
		{
			fld = TblGetCurrentField (table);
			str = FldGetTextPtr (fld);
		}
		else
		{
			index = FieldMap[fieldIndex];
			str = record->field[index];
		}
	}
	else
	{
		index = FieldMap[fieldIndex];
		str = record->field[index];
	}

	FntSetFont(stdFont);
	height = FldCalcFieldHeight (str, columnWidth);
	lineHeight = FntLineHeight ();
	height = min (height, (maxHeight / lineHeight));
	height *= lineHeight;

	return (height);
}

void EditInitTableRow( FormType* frmP, TablePtr table, UInt16 row, UInt16 fieldIndex, 
	Int16 rowHeight, WordRecordType* record)
{
	
	// Make the row usable.
	TblSetRowUsable (table, row, true);

	// Set the height of the row to the height of the desc
	TblSetRowHeight (table, row, rowHeight);

	// Store the record number as the row id.
	TblSetRowID (table, row, fieldIndex);

	// Mark the row invalid so that it will draw when we call the
	// draw routine.
	TblMarkRowInvalid (table, row);
	
	TblSetItemPtr(table, row, labelColumn, fieldLabel[FieldMap[fieldIndex]]);
	
	if (FieldMap[fieldIndex] == WordFieldPhonetic){
		TblSetItemStyle (table, row, dataColumn, customTableItem);
	}
	else{
		TblSetItemStyle (table, row, dataColumn, textTableItem);
	}
}

void EditFormNextWord(Boolean next){
	UInt16 qIndex, num;
	HeaderItem item;
	Int16 direction;
	
	switch (tabCurrentSelected){
		
		case StudyListTab:
		
			if (!FindHeaderItemByDBIndex(CurrentDB, LearningRecordIndex, CurrentRecord, &qIndex))
				return;
			
			num = GetHeaderItemNumber(CurrentDB, LearningRecordIndex);
			
			if (next){
				
				if (qIndex == num - 1) return;
				
				qIndex ++;
				
			}
			
			else{
			
				if (qIndex == 0) return;
				
				qIndex --;
							
			}
	
			GetHeaderItem(CurrentDB, LearningRecordIndex, &item, qIndex);
				
			DmFindRecordByID(CurrentDB, item, &CurrentRecord);
			
			
			break;
			
		case RawTab:
		case AllTab:
			if (next) {
				direction = dmSeekForward;
				qIndex = CurrentRecord + 1;
			}
			else {
				direction = dmSeekBackward;
				if (CurrentRecord == FirstWordRecord) return;
				qIndex = CurrentRecord - 1;
			}
			
			if (!SeekNextRecord(&qIndex, direction)) return;
			
			CurrentRecord = qIndex;
			break;
			
		default:
			break;

	
	}

	EditFormLoadControls();
	EditFormLoadCategory();
	EditFormLoadTable(FrmGetActiveForm());
	FrmDrawForm(FrmGetActiveForm());
	EditUpdateHorizonScrollers();

}

void EditUpdateScrollers (FormPtr frmP, UInt16 bottomFieldIndex, Boolean lastItemClipped){

	TableType * table = GetObjectPtr(EditEditTable);
	UInt16 upIndex;
	UInt16 downIndex;
	Boolean scrollableUp = false;
	Boolean scrollableDown = false;
//	Boolean 
	
	// If the first field displayed is not the fist field in the record,
	// enable the up scroller.
	
	if (TopVisibleFieldIndex > 0) scrollableUp = true;
	if (lastItemClipped || (bottomFieldIndex < MaxFieldIndex)) scrollableDown = true;

	// If the last field displayed is not the last field in the record,
	// enable the down scroller.

	// Update the scroll button.
	upIndex = FrmGetObjectIndex (frmP, EditScrollUpRepeating);
	downIndex = FrmGetObjectIndex (frmP, EditScrollDownRepeating);
	FrmUpdateScrollers (frmP, upIndex, downIndex, scrollableUp, scrollableDown);
}

void EditUpdateHorizonScrollers(){
	UInt16 qIndex, num;
	
	FormType* frmP = FrmGetActiveForm();
	Boolean scrollableUp = true;
	Boolean scrollableDown = true;
	
	switch(tabCurrentSelected){
		case StudyListTab:
			
			FindHeaderItemByDBIndex(CurrentDB, LearningRecordIndex, CurrentRecord, &qIndex);
			num = GetHeaderItemNumber(CurrentDB, LearningRecordIndex);
			
			if (qIndex == 0) scrollableUp = false;
			if (qIndex == num - 1) scrollableDown = false;
		
			break;
		
		case RawTab:
		case AllTab:
			if (CurrentRecord == FirstWordRecord) scrollableUp = false;
			
			else {
				qIndex = CurrentRecord - 1;
				if (!SeekNextRecord(&qIndex, dmSeekBackward)) scrollableUp = false;
			}
			
			qIndex = CurrentRecord + 1;
			if (!SeekNextRecord(&qIndex, dmSeekForward)) scrollableDown = false;
			break;		
	
	}

	if (scrollableUp)	ShowObject(EditPrevRepeating);
	else	HideObject(EditPrevRepeating);
	
	if (scrollableDown)	ShowObject(EditNextRepeating);
	else	HideObject(EditNextRepeating);

}

UInt16 EditFormSaveRecord ()
{
	MemHandle currentRecordH;
	WordRecordType currentRecord;
	FormPtr frmP;
	TablePtr tableP;
	Boolean hasData;


	// Make sure the field being edited is saved
	frmP = FrmGetFormPtr(EditForm);
	tableP = FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, EditEditTable));

	TblReleaseFocus(tableP);


	// The record may have already been delete by the Delete menu command
	// or the details dialog.  If there isn't a CurrentRecord assume the
	// record has been deleted.
	if (CurrentRecord == noRecord)
	{
		return BrowseForm;
	}

	// If there is no data then then delete the record.
	// If there is data but no name data then demand some.

	GetWordRecord(CurrentDB, CurrentRecord, &currentRecord, &currentRecordH);

	hasData = WordRecordContainsData(&currentRecord);

	// Unlock before the DeleteRecord.   We can only rely on
	// NULL pointers from here on out.
	MemHandleUnlock(currentRecordH);


	// If none are the fields contained anything then
	// delete the field.
	if (!hasData)
	{
		WordDBDeleteRecord(false);   // uniq ID wasted?  Yes. We don't care.
		return BrowseForm;
	}


	// The record's category may have been changed.  The CurrentCategory
	// isn't supposed to change in this case.  Make sure the CurrentRecord
	// is still visible in this category or pick another one near it.
	//if (!SeekRecord(&CurrentRecord, 0, dmSeekBackward))
		//if (!SeekRecord(&CurrentRecord, 0, dmSeekForward))
			//CurrentRecord = noRecord;



	return BrowseForm;
}

void EditNewRecord ()
{
	WordRecordType newRecord;
	WordFields i;
	UInt16 attr;
	Err err;
	MemorizingStatus memoStatus ={2.50, //e-factor
		40, //difficulty 
		0, //total lapse
		0, //total recall
		0, //recent lapse
		0, //recent recall
		0, //step
		0, //last test
		0, //current test
		0,	//next test
		0,	//first learn
		0}; //create

	DateType today;
	UInt32 seconds;
	
	newRecord.header.allBits = 0;
	SetBitMacro(newRecord.header.allBits, RawQueue);
	
	newRecord.ltStatus.learningStatus = 0;
	newRecord.memoStatus = memoStatus;
	
	seconds = TimGetSeconds();
	DateSecondsToDate(seconds, &today);
	newRecord.memoStatus.create = today;
	
	for (i = WordFieldQuestion; i < WordFieldsCount; i++)
		newRecord.field[i] = NULL;

	err = NewWordRecord(CurrentDB, &newRecord, &CurrentRecord);
	if (err)
	{
		FrmAlert(DeviceFullAlert);
		return;
	}

	// Set it's category to the category being viewed.
	// If the category is All then set the category to unfiled.
	DmRecordInfo (CurrentDB, CurrentRecord, &attr, NULL, NULL);
	attr &= ~dmRecAttrCategoryMask;
	attr |= ((CurrentCategory == dmAllCategories) ? dmUnfiledCategory :
			 CurrentCategory) | dmRecAttrDirty;
	DmSetRecordInfo (CurrentDB, CurrentRecord, &attr, NULL);

	IncrementRawWordCount();
	// Set the global variable that determines which field is the top visible
	// field in the edit view.  Also done when New is pressed.
	TopVisibleFieldIndex = 0;
	CurrentFieldIndex = 0;
	EditRowIDWhichHadFocus = 0;
	EditFieldPosition = 0;

	FrmGotoForm(EditForm);
}

void EditFormScroll (WinDirectionType direction)
{
	
	UInt16				row;
	UInt16				height;
	UInt16				fieldIndex;
	UInt16				columnWidth;
	UInt16				tableHeight;
	TablePtr			table;
	//FontID				curFont;
	RectangleType		r;
	WordRecordType	record;
	MemHandle			recordH;
	FormType*			frmP;


	// Before processing the scroll, be sure that the command bar has been closed.
	MenuEraseStatus (0);
	frmP = FrmGetActiveForm();

	FntSetFont (stdFont);

	table = GetObjectPtr(EditEditTable);  
	TblReleaseFocus (table);

	// Get the height of the table and the width of the description
	// column.
	TblGetBounds (table, &r);
	tableHeight = r.extent.y;
	height = 0;
	columnWidth = TblGetColumnWidth (table, dataColumn);

	// Scroll the table down.
	if (direction == winDown)
	{
		// Get the index of the last visible field, this will become
		// the index of the top visible field, unless it occupies the
		// whole screeen, in which case the next field will be the
		// top filed.

		row = TblGetLastUsableRow (table);
		fieldIndex = TblGetRowID (table, row);

		// If the last visible field is also the first visible field
		// then it occupies the whole screeen.
		if (row == 0)
			fieldIndex = min (MaxFieldIndex, fieldIndex+1);
	}

	// Scroll the table up.
	else
	{
		// Scan the fields before the first visible field to determine
		// how many fields we need to scroll.  Since the heights of the
		// fields vary,  we sum the height of the records until we get
		// a screen full.

		fieldIndex = TblGetRowID (table, 0);
		ErrFatalDisplayIf(fieldIndex > MaxFieldIndex, "Invalid field Index");
		if (fieldIndex == 0)
			goto exit;

		// Get the current record
		GetWordRecord (CurrentDB, CurrentRecord, &record, &recordH);

		height = TblGetRowHeight (table, 0);
		if (height >= tableHeight)
			height = 0;

		while (height < tableHeight && fieldIndex > 0)
		{
			height += FldCalcFieldHeight (record.field[FieldMap[fieldIndex-1]],
										  columnWidth) * FntLineHeight ();
			if ((height <= tableHeight) || (fieldIndex == TblGetRowID (table, 0)))
				fieldIndex--;
		}
		MemHandleUnlock(recordH);
	}

	TblMarkTableInvalid (table);
	CurrentFieldIndex = noFieldIndex;
	TopVisibleFieldIndex = fieldIndex;
	EditRowIDWhichHadFocus = FirstFieldIndex;
	EditFieldPosition = 0;

	TblUnhighlightSelection (table);		// remove the highlight before reloading the table to avoid
	// having an out of bounds selection information in case
	// the newly loaded data doesn't have as many rows as the old
	// data.  This fixes the bug
	// "File: Table.c, Line: 2599, currentRow violated constraint!"
	// (fix suggested by Art) vmk 2/20/98
	EditFormLoadTable(frmP);

	//TblUnhighlightSelection (table);	// moved call before PrvEditLoadTable vmk 2/20/98
	TblRedrawTable (table);

	exit:
		FntSetFont (stdFont);
}

void EditFormResizeField (EventType * event)
{
	UInt16 pos;
	Int16 row;
	Int16 column;
	Int16 lastRow;
	UInt16 fieldIndex;
	UInt16 lastFieldIndex;
	UInt16 topFieldIndex;
	FieldPtr fld;
	TablePtr table;
	Boolean restoreFocus = false;
	Boolean lastItemClipped;
	RectangleType itemR;
	RectangleType tableR;
	RectangleType fieldR;
	FormType* frmP;


	frmP = FrmGetActiveForm();
	// Get the current height of the field;
	fld = event->data.fldHeightChanged.pField;
	FldGetBounds (fld, &fieldR);

	// Have the table object resize the field and move the items below
	// the field up or down.
	table = FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, EditEditTable));
	TblHandleEvent (table, event);

	// If the field's height has expanded , we're done.
	if (event->data.fldHeightChanged.newHeight >= fieldR.extent.y)
	{
		topFieldIndex = TblGetRowID (table, 0);
		if (topFieldIndex != TopVisibleFieldIndex)
			TopVisibleFieldIndex = topFieldIndex;
		else
		{
			// Since the table has expanded we may be able to scroll
			// when before we might not have.
			lastRow = TblGetLastUsableRow (table);
			TblGetBounds (table, &tableR);
			TblGetItemBounds (table, lastRow, dataColumn, &itemR);
			lastItemClipped = (itemR.topLeft.y + itemR.extent.y >
							   tableR.topLeft.y + tableR.extent.y);
			lastFieldIndex = TblGetRowID (table, lastRow);

			EditUpdateScrollers(frmP, lastFieldIndex, lastItemClipped);

			return;
		}
	}

	// If the field's height has contracted and the field edit field
	// is not visible then the table may be scrolled.  Release the
	// focus,  which will force the saving of the field we are editing.
	else if (TblGetRowID (table, 0) != FirstFieldIndex)
	{
		TblGetSelection (table, &row, &column);
		fieldIndex = TblGetRowID (table, row);

		fld = TblGetCurrentField (table);
		pos = FldGetInsPtPosition (fld);
		TblReleaseFocus (table);

		restoreFocus = true;
	}

	// Add items to the table to fill in the space made available by the
	// shorting the field.
	EditFormLoadTable(frmP);
	TblRedrawTable (table);

	// Restore the insertion point position.
	if (restoreFocus)
	{
		TblFindRowID (table, fieldIndex, &row);
		TblGrabFocus (table, row, column);
		FldSetInsPtPosition (fld, pos);
		FldGrabFocus (fld);
	}
}

UInt16 EditCategorySelection (void)
{
	UInt16 attr;
	FormType* frmP;
	UInt16 category;
	Boolean categoryEdited;


	// Process the category popup list.
	DmRecordInfo (CurrentDB, CurrentRecord, &attr, NULL, NULL);
	category = attr & dmRecAttrCategoryMask;

	frmP = FrmGetActiveForm();
	categoryEdited = CategorySelect (CurrentDB, frmP, EditCategoryPopTrigger, EditWordCategoryList,
			false, &category, EditCategoryName, 1, categoryDefaultEditCategoryString);
	
	if (categoryEdited || (category != (attr & dmRecAttrCategoryMask)))
	{
		// Change the category of the record.
		DmRecordInfo (CurrentDB, CurrentRecord, &attr, NULL, NULL);
		attr &= ~dmRecAttrCategoryMask;
		attr |= category | dmRecAttrDirty;
		DmSetRecordInfo (CurrentDB, CurrentRecord, &attr, NULL);

		//ToolsChangeCategory (category);
	}
	return (category);
}

void FieldSelection(Int16 row){
	FormType * frmP;
	TableType * tableP;
	
	frmP = FrmGetActiveForm();
	tableP = FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, EditEditTable));
		
	CurrentFieldIndex = TblGetRowID(tableP, row);
	if (FieldMap[TblGetRowID(tableP, row)] == WordFieldPhonetic){
		TblUnhighlightSelection(tableP);
		FrmPopupForm(PhoneticForm);
	}
}

void EditDrawRecordField (void * table, Int16 row, Int16 column, RectanglePtr bounds){
	WordRecordType record;
	MemHandle recordH;
	UInt16 fieldIndex;
	FontID currFont;
	
	fieldIndex = TblGetRowID(table, row);
	if (FieldMap[fieldIndex] == WordFieldPhonetic){
		
		GetWordRecord(CurrentDB, CurrentRecord, &record, &recordH);
		
		if (record.field[WordFieldPhonetic]!=NULL){
			currFont = FntSetFont(PhoneticSmallFont);
			WinDrawChars(record.field[WordFieldPhonetic], StrLen(record.field[WordFieldPhonetic]),
				bounds->topLeft.x, bounds->topLeft.y);
			FntSetFont(currFont);
		}
		
		MemHandleUnlock(recordH);
		return;
	}
}

static void PhoneticInit(FormType * frmP);
static void DrawVowel (void * table, Int16 row, Int16 column, RectanglePtr bounds);
static void DrawConsonant (void * table, Int16 row, Int16 column, RectanglePtr bounds);
static void SavePhonetic();

Boolean PhoneticFormHandleEvent(EventType * event){

	FormType* frmP = FrmGetActiveForm();
	FieldType* fldP = FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, PhoneticInputField));
	Char buffer[5];
	Char* p;
	Boolean handled = false;
	
	switch (event->eType){
		
		case frmOpenEvent:
			PhoneticInit (frmP);
			FrmDrawForm (frmP);
			
			handled = true;
			break;
			
		case ctlSelectEvent:
			switch(event->data.ctlSelect.controlID){
				case PhoneticOKButton:
					SavePhonetic();
					FrmReturnToForm(EditForm);
					FrmUpdateForm(EditForm, frmRedrawUpdateCode);
					handled = true;
					break;
				
				case PhoneticCancelButton:
					FrmReturnToForm(EditForm);
					FrmUpdateForm(EditForm, frmRedrawUpdateCode);
					handled = true;
					break;
					
				default:
					break;
			
			}
			break;
		
		case tblSelectEvent:
			switch(event->data.tblSelect.tableID){
				case PhoneticVowelTable:
					TblUnhighlightSelection(event->data.tblSelect.pTable);
					p = SysStringByIndex(VowelsStringList, 
						event->data.tblSelect.row * ColumnsInVowelTable 
						+ event->data.tblSelect.column, buffer, 4);
					FldInsert(fldP, p, StrLen(p));
					break;
					
				case PhoneticConsonantTable:
					TblUnhighlightSelection(event->data.tblSelect.pTable);
					p = SysStringByIndex(ConsonantsStringList, 
						event->data.tblSelect.row * ColumnsInConsonantTable 
						+ event->data.tblSelect.column, buffer, 4);
					FldInsert(fldP, p, StrLen(p));
					break;
			
			}
			
			FrmSetFocus(frmP, FrmGetObjectIndex(frmP, PhoneticInputField));			
			FldGrabFocus(fldP);
			handled = true;
			break;
		
		default:
			break;
	}
	
	return handled;
}

void PhoneticInit(FormType * frmP){
	
	TableType * table;
	UInt16 row, column;
	UInt16 rowsInTable;
	
	WordRecordType record;
	MemHandle recordH;
	MemHandle textH, oldTextH;
	Char*	p;
	UInt16 textAllocSize;
	FieldType* fld;
	
	table = FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP,PhoneticVowelTable));
	rowsInTable = TblGetNumberOfRows (table);
	
	for (row = 0; row < rowsInTable; row++)
	{
		for (column = 0; column < ColumnsInVowelTable; column ++)
			TblSetItemStyle (table, row,  column, customTableItem);

		TblSetRowUsable (table, row, true);
	}
	
	for (column = 0; column < ColumnsInVowelTable; column ++){
		TblSetColumnUsable (table, column, true);
		TblSetCustomDrawProcedure (table, column, DrawVowel);
	}

	table = FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP,PhoneticConsonantTable));
	rowsInTable = TblGetNumberOfRows (table);
	
	for (row = 0; row < rowsInTable; row++)
	{
		for (column = 0; column < ColumnsInVowelTable; column ++)
			TblSetItemStyle (table, row,  column, customTableItem);

		TblSetRowUsable (table, row, true);
	}
	
	for (column = 0; column < ColumnsInConsonantTable; column ++){
		TblSetColumnUsable (table, column, true);
		TblSetCustomDrawProcedure (table, column, DrawConsonant);
	}
	
	//Get the record
	GetWordRecord(CurrentDB, CurrentRecord, &record, &recordH); //recordH locked
	
	//Calculate mem handle size
	if (record.field[WordFieldPhonetic] != NULL)
		textAllocSize = StrLen(record.field[WordFieldPhonetic]) + 1;	//for null terminator
	else
		textAllocSize = 1;
	
	//Allocate textH
	textH = MemHandleNew(textAllocSize);
	p = MemHandleLock(textH);	//textH locked
	
	//clear it
	MemSet(p, textAllocSize, 0);
	
	//copy 
	MemMove(p, record.field[WordFieldPhonetic], textAllocSize - 1);
	
	//unlock
	MemHandleUnlock(recordH);
	MemHandleUnlock(textH);
	
	fld = FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, PhoneticInputField));
	
	oldTextH = FldGetTextHandle(fld);
	FldSetTextHandle(fld, textH);
	
	if (oldTextH) MemHandleFree(oldTextH);
	
	//set font
	FldSetFont(fld, PhoneticSmallFont);
}

void DrawVowel (void * table, Int16 row, Int16 column, RectanglePtr bounds){

	Char buffer[5];
	Char* p;
	FontID currFont;
	UInt16 width, height;
	
	currFont = FntSetFont(PhoneticSmallFont);
	p = SysStringByIndex(VowelsStringList, row * ColumnsInVowelTable + column, buffer, 4);
		
	if (p!=NULL){
		
		width = FntCharsWidth(p, StrLen(p));
		height = FntLineHeight();	
		
		WinDrawChars(p, StrLen(p), bounds->topLeft.x + (bounds->extent.x - width)/2, 
			bounds->topLeft.y + (bounds->extent.y - height)/2); 
	}
	FntSetFont(currFont);	

}

void DrawConsonant (void * table, Int16 row, Int16 column, RectanglePtr bounds){

	Char buffer[5];
	Char* p;
	FontID currFont;
	UInt16 width, height;
	
	currFont = FntSetFont(PhoneticSmallFont);
	p = SysStringByIndex(ConsonantsStringList, row * ColumnsInConsonantTable + column, buffer, 4);
		
	if (p!=NULL){
		
		width = FntCharsWidth(p, StrLen(p));
		height = FntLineHeight();	
		
		WinDrawChars(p, StrLen(p), bounds->topLeft.x + (bounds->extent.x - width)/2, 
			bounds->topLeft.y + (bounds->extent.y - height)/2); 
	}

	FntSetFont(currFont);	

}

void SavePhonetic(){

	FormType* frmP;
	FieldType * fld;
	MemHandle textH;
	Char* textP;
	
	WordRecordType record;
	MemHandle recordH;
	WordFieldFlags flag;
	
	Err err;
	
	//get field
	frmP = FrmGetActiveForm();
	fld = FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, PhoneticInputField));
	
	//if no change, directly return
	if (!FldDirty(fld)) return;
	
	//get text handle
	textH = FldGetTextHandle(fld);
	
	//remove selection 
	FldSetSelection(fld, 0, 0);
	
	//set pointer	
	if (textH == 0)
		textP = NULL;
	else {
		textP = MemHandleLock(textH);			//lock once
		if (textP[0] == '\0') textP = NULL;
	}
	
	//get record
	GetWordRecord(CurrentDB, CurrentRecord, &record, &recordH);
	
	//change field
	record.field[WordFieldPhonetic] = textP;
	
	//set change flag
	flag.allBits = BitAtPosition(WordFieldPhonetic);
	
	//save changes
	err = ChangeWordRecord(CurrentDB, &CurrentRecord, &record, flag);
	
	//seems different from that of .... palm code
	if (textH != 0) MemHandleUnlock(textH);
	
	//deal with err
	if (err){
		ErrNonFatalDisplay("Out of Memory");
		MemHandleUnlock(recordH);
		return;
	}
	
	ToolsDirtyRecord(CurrentRecord);
}

