#include <PalmOS.h>
#include "SugarMemo.h"
#include "SugarMemoRsc.h"
#include "SugarMemoLearn.h"
#include "SugarMemoDB.h"
#include "SugarMemoTools.h"

#define ShowNext 				1
#define ShowShowAnswer 			2
#define ShowRightWrong 			12
#define ShowBingoPassFail 		112

#define	DynamicPushButtonStart	6666
#define	DynamicPushButtonGroup	1234

#define RightWrongGroup 	1
#define BingoPassFailGroup 	2

typedef union{
	struct{
		unsigned reserved	:9;
		unsigned fail 	:1;	//64
		unsigned pass	:1;	//32
		unsigned bingo	:1;	//16
		unsigned wrong	:1;	//8
		unsigned right 	:1;	//4
		unsigned show 	:1;	//2
		unsigned next 	:1;	//1
	} bits;
	UInt16 allBits;
}	ControlFlags;

typedef enum{
	nextControl = 0,
	showControl,
	rightControl,
	wrongControl,
	bingoControl,
	passControl,
	failControl,
	controlCounts,
	noSelectedControl = 0xFFFF
} Control;

UInt16		CurrentAnswerTop;
UInt16 		CurrentTop = 0;
UInt16 		CurrentHeight;
UInt16 		maxQuestionPaneHeight = 80;
Boolean		ShowAnswerPane = false;

Control 	PushSelection;

ControlType*	fieldTab[WordFieldsCount];
WordFields		CurrentAnswerField = WordFieldAnswer;
WordFields 		DefaultAnswerField = WordFieldAnswer;

RectangleType 	QuestionPane;
RectangleType 	AnswerPane;
RectangleType	ToolBar = {{0, 145}, {160, 15}};

Boolean Lefty = false;
WordFieldFlags 	fieldFlag;
ControlFlags 	controlFlag;

extern EditLabel fieldLabel[WordFieldsCount];

static void 	RemoveBlankHead(char** p);
static void 	DrawFieldHeight(char* p, UInt16* height, TextAlignment align, Boolean draw);
static UInt16 	DrawPaneHeight(Boolean draw);
static void 	ScrollCanvas(Boolean up);
static void 	DrawBanner();
static void		DrawCanvas();
static void		DrawQuestionPane();
static void		DrawAnswerPane();
static void 	LoadAnswerTab();
static void		ClearAnswerTab();

static void 	UpdateScrollers();
static void 	LoadControls();
static UInt16 	GetQuestionPageFields();
static UInt16 	GetAnswerPageFields();

static void 	DoControlShowButton();
static void 	DoControlNext();
static UInt8 	Difficulty(float efactor);
static RectangleType GetCanvasBounds();
static Boolean 	LoadWord();
static void 	Finish();
static void 	SwitchButtonOff(UInt16 controlID);
static void 	SwitchButtonOn(UInt16 controlID);
static Boolean 	LearnFormDoCommand(UInt16 command);

static void 	DrawCanvas(){
	DrawQuestionPane();

	WinEraseRectangle(&AnswerPane, 0);
	
	WinPushDrawState();
	WinSetForeColor(UIColorGetTableEntryIndex(UIObjectSelectedFill));
	WinDrawGrayLine(0, AnswerPane.topLeft.y - 3 , 160, AnswerPane.topLeft.y - 3);
	WinPopDrawState();
	
	FrmUpdateScrollers(FrmGetActiveForm(), GetObjectIndex(LearnScrollUpRepeating),
		GetObjectIndex(LearnScrollDownRepeating), false, false);
	

	if (ShowAnswerPane)	{
		CurrentAnswerField = DefaultAnswerField;
		LoadAnswerTab();
		
		FrmDrawForm(FrmGetActiveForm());
		DrawAnswerPane();
	}
}

static void 	DrawQuestionPane(){

	WinHandle activeWinH, offscreenWinH;
	RectangleType bounds, questionPane;
	Err error;
	
	UInt16 height;
	
	SugarAppInfoType* app;
	
	app = GetAppInfoPtr(CurrentDB);
	fieldFlag = app -> 	questionPageFields;
	MemPtrUnlock(app);
	
	height = DrawPaneHeight(false);
	if (height > maxQuestionPaneHeight) height = maxQuestionPaneHeight;
	
	//save current active window handle
	activeWinH = WinGetActiveWindow();
	
	//create offscreen window and set as draw window
	questionPane = GetCanvasBounds();
	questionPane.extent.y = height;
	offscreenWinH = WinCreateOffscreenWindow(questionPane.extent.x, questionPane.extent.y, genericFormat, &error);
	
	if (error) ErrAlert(error);
	WinSetDrawWindow(offscreenWinH);
	
	//draw the window
	DrawPaneHeight(true);

	//change the draw window
	WinSetDrawWindow(activeWinH);
	
	//blit
	bounds.topLeft.x = 0;
	bounds.topLeft.y = 0;
	bounds.extent.x = questionPane.extent.x;
	bounds.extent.y = questionPane.extent.y;
	WinCopyRectangle(offscreenWinH, activeWinH, &bounds, questionPane.topLeft.x, questionPane.topLeft.y, winPaint);
	
	//delete offscreen window
	WinDeleteWindow(offscreenWinH, false);
	
	AnswerPane.topLeft.x = questionPane.topLeft.x;
	AnswerPane.topLeft.y = questionPane.topLeft.y + questionPane.extent.y;
	AnswerPane.extent.x = questionPane.extent.x;
	AnswerPane.extent.y = GetCanvasBounds().extent.y - questionPane.extent.y;
}

static void 	DrawAnswerPane(){
	
	WinHandle activeWinH, offscreenWinH;
	RectangleType bounds;
	Err error;
	FontID currFont;
	
	SugarAppInfoType* app;
	MemHandle recordH;
	WordRecordType record;
	UInt16 top = 0;

	UInt16 upIndex, downIndex;
	Boolean Scrollup = false;
	Boolean Scrolldown = false;


	//	Get App Info Pointer	
	app = GetAppInfoPtr(CurrentDB);
	GetWordRecord(CurrentDB, CurrentRecord, &record, &recordH);

	currFont = FntSetFont(app -> learnFieldFont[CurrentAnswerField]);
	DrawFieldHeight(record.field[CurrentAnswerField], &top, app->learnFieldAlignment[CurrentAnswerField], false);
	
	if (top <= AnswerPane.extent.y) {
		CurrentAnswerTop = (AnswerPane.extent.y - top) / 2;
	}
	
	//save current active window handle
	activeWinH = WinGetActiveWindow();
	
	//create offscreen window and set as draw window
	offscreenWinH = WinCreateOffscreenWindow(AnswerPane.extent.x, AnswerPane.extent.y, genericFormat, &error);
	
	if (error) ErrAlert(error);
	WinSetDrawWindow(offscreenWinH);
	
	//draw the window
	top = CurrentAnswerTop;
	DrawFieldHeight(record.field[CurrentAnswerField], &top, app->learnFieldAlignment[CurrentAnswerField], true);
	
	//	release pointer
	MemHandleUnlock(recordH);
	MemPtrUnlock(app);
	FntSetFont(currFont);

	//change the draw window
	WinSetDrawWindow(activeWinH);
	
	//blit
	bounds.topLeft.x = 0;
	bounds.topLeft.y = 0;
	bounds.extent.x = AnswerPane.extent.x;
	bounds.extent.y = AnswerPane.extent.y;
	WinCopyRectangle(offscreenWinH, activeWinH, &bounds, AnswerPane.topLeft.x, AnswerPane.topLeft.y, winPaint);
	
	//delete offscreen window
	WinDeleteWindow(offscreenWinH, false);
	
	if (CurrentAnswerTop > 160)
		Scrollup = true;

	if (top > AnswerPane.extent.y)
		Scrolldown = true;
	
	upIndex = GetObjectIndex(LearnScrollUpRepeating);
	downIndex = GetObjectIndex(LearnScrollDownRepeating);
	
	FrmUpdateScrollers(FrmGetActiveForm(), upIndex, downIndex, Scrollup, Scrolldown);
}

Boolean LearnFormHandleEvent(EventType * event){

	FormType* frmP = FrmGetActiveForm();
	UInt16 gadgetIndex;
	RectangleType bounds;
	Boolean handled = false;
	
	switch (event->eType){
		
		case frmOpenEvent:
			
			if (LoadWord()){
				LoadControls();
				DrawBanner();
				FrmDrawForm(frmP);			
				DrawCanvas();
			}
			handled = true;
			break;
			
		case frmUpdateEvent:
			LoadControls();
			FrmDrawForm(frmP);
			DrawCanvas();
			handled = true;
			break;			
			
		case menuEvent:
            return LearnFormDoCommand(event->data.menu.itemID);			
			
		case penDownEvent:
			gadgetIndex = FrmGetObjectIndex(frmP, LearnCanvasGadget);
			FrmGetObjectBounds(frmP,gadgetIndex, &bounds);
			if (RctPtInRectangle(event->screenX, event->screenY, &bounds)){
				SndPlaySystemSound(sndClick);
				FrmGotoForm(EditForm);
				handled = true;
				break;
			}
						
		case ctlSelectEvent:
			if (event->data.ctlSelect.controlID - DynamicPushButtonStart < WordFieldsCount){
				CurrentAnswerField = event->data.ctlSelect.controlID - DynamicPushButtonStart;
				CurrentAnswerTop = 0;
				DrawAnswerPane();
				handled = true;
				break;
			}
			switch (event->data.ctlSelect.controlID){
								
				case LearnShowButton:
					DoControlShowButton();
					handled = true;
					break;
					
				case LearnNextButton:
					DoControlNext();
					handled = true;
					break;
					
				case LearnRightPushButton:
					PushSelection = rightControl;
					DoControlNext();
					break;
					
				case LearnWrongPushButton:
					PushSelection = wrongControl;
					DoControlNext();
					break;
					
				case LearnBingoPushButton:
					PushSelection = bingoControl;
					DoControlNext();
					break;
					
				case LearnPassPushButton:
					PushSelection = passControl;
					DoControlNext();
					break;
					
				case LearnFailPushButton:
					PushSelection = failControl;
					DoControlNext();
					break;
					
				
				default:
					break;
			
			}
			break;
		
		case ctlRepeatEvent:
			switch(event->data.ctlRepeat.controlID){
				case LearnScrollUpRepeating:
					ScrollCanvas(true);
					break;
				case LearnScrollDownRepeating:
					ScrollCanvas(false);
					break;
			
			}
			break;
			
		case keyDownEvent:

			if (EvtKeydownIsVirtual(event))
			{
				switch (event->data.keyDown.chr)
				{
					case vchrPageUp:
						ScrollCanvas (true);
						handled = true;
						break;
		
					case vchrPageDown:
						ScrollCanvas (false);
						handled = true;
						break;
				}
			}
			break;
			
		default:
			break;
	}
	
	return handled;
}

Boolean LearnFormDoCommand(UInt16 command){
	
	FormType* frmP = FrmGetActiveForm();
	Boolean handled = false;
	
	switch(command){
//		case OptionQuit:
//			MenuEraseStatus(0);
			
//			if (CurrentDB != NULL) DmCloseDatabase(CurrentDB);
//			HeaderRecordIndex = HeaderIndexCounts; 
//			FrmGotoForm(MainForm);
//			handled = true;
//			break;
		
		default:
			break;
	}
	return handled;
}

UInt16 DrawPaneHeight(Boolean draw){
	FontID currFont;
	UInt16 top = 0;
	WordRecordType record;
	MemHandle recordH;
	WordFields i;
	char* p;
	SugarAppInfoType * appInfo;
	
	appInfo = GetAppInfoPtr(CurrentDB);
	currFont = FntGetFont();
	GetWordRecord(CurrentDB, CurrentRecord, &record, &recordH);
	for (i = WordFieldQuestion; i < WordFieldsCount; i++){
		if (GetBitMacro(fieldFlag.allBits, i) != 0 && record.field[i]!= NULL){
			top += 3;
			p = record.field[i];
			FntSetFont(appInfo -> learnFieldFont[i]);
			DrawFieldHeight(p, &top, appInfo -> learnFieldAlignment[i], draw);
		}
	}
	MemHandleUnlock(recordH);
	
	MemPtrUnlock(appInfo);
	FntSetFont(currFont);
	
	top += 6;
	return top;
}

void DrawFieldHeight(char* p, UInt16* top, TextAlignment align, Boolean draw){

	Int16 strWidth;
	Int16 strLength;
	Int16 xpos;
	char* p2;
	
	while (true){
		RemoveBlankHead(&p);
		if ((*p) == 0) break;
		strWidth = 160;
		strLength = StrLen(p);
		//FntCharsInWidth(p, &strWidth, &strLength, &fit);
		strLength = FntWordWrap(p, 160);
		strWidth = FntCharsWidth(p, strLength);
		
		p2 = p + strLength - 1;
		if ((*p2) == 9 || (*p2) == 10 || (*p2) == 13 || (*p2) == 32)
			strLength--;
		
		if (draw){
			switch (align){
				case LeftAlignment:
					xpos = 0;
					break;
				case CenterAlignment:
					xpos = (160 - strWidth)/2;
					break;
				case RightAlignment:
					xpos = 160 - strWidth;
					break;
			}
			WinDrawChars(p, strLength, xpos, *top); 
		}
		p += strLength;
		(*top) += FntLineHeight();
	}
}

void RemoveBlankHead(char** p){
	while ((**p) == 9 			//Horizontal Tabulation
			|| (**p) == 10		//Line Feed
			|| (**p) == 13		//Carrige Return
			|| (**p) == 32)		//Space
		(*p)++;
}

void UpdateScrollers(){
	
	UInt16 upIndex, downIndex;
	Boolean Scrollup = false;
	Boolean Scrolldown = false;
	FormType* frmP = FrmGetActiveForm();
	RectangleType bounds = GetCanvasBounds();
	
	if (CurrentTop < bounds.topLeft.y || CurrentTop > 160)
		Scrollup = true;

	if (CurrentTop + CurrentHeight > bounds.topLeft.y + bounds.extent.y)
		Scrolldown = true;
	
	upIndex = FrmGetObjectIndex(frmP, LearnScrollUpRepeating);
	downIndex = FrmGetObjectIndex(frmP, LearnScrollDownRepeating);
	
	FrmUpdateScrollers(frmP, upIndex, downIndex, Scrollup, Scrolldown);
}

void ScrollCanvas(Boolean up){

	SugarAppInfoType* app;
	WordRecordType record;
	MemHandle recordH;
	UInt16 height = 0;
	FontID font;
	
	app = GetAppInfoPtr(CurrentDB);
	GetWordRecord(CurrentDB, CurrentRecord, &record, &recordH);
	
	font = FntSetFont(app->learnFieldFont[CurrentAnswerField]);
	DrawFieldHeight(record.field[CurrentAnswerField], &height, app->learnFieldAlignment[CurrentAnswerField], false);
	
	MemPtrUnlock(app);
	MemHandleUnlock(recordH);
	
	if (up){
		if (CurrentAnswerTop + AnswerPane.extent.y - FntLineHeight() > AnswerPane.extent.y)
			CurrentAnswerTop += AnswerPane.extent.y - FntLineHeight();
		else
			CurrentAnswerTop = 0; 
	}
	else{
		if (CurrentAnswerTop + height - AnswerPane.extent.y > AnswerPane.extent.y - FntLineHeight())
			CurrentAnswerTop -= AnswerPane.extent.y - FntLineHeight();
		else
			CurrentAnswerTop = AnswerPane.extent.y - height;
	}

	DrawAnswerPane();	
}

void LoadControls(){

	FormType * frmP = FrmGetActiveForm();
	
	if (GetBitMacro(controlFlag.allBits, nextControl)!= 0) ShowObject(LearnNextButton);
	else	HideObject(LearnNextButton);
	
	//show answer button
	if (GetBitMacro(controlFlag.allBits, showControl)!= 0) ShowObject(LearnShowButton);
	else	HideObject(LearnShowButton);
	
	//right push
	if (GetBitMacro(controlFlag.allBits, rightControl)!= 0) ShowObject(LearnRightPushButton);
	else	HideObject(LearnRightPushButton);

	//wrong push
	if (GetBitMacro(controlFlag.allBits, wrongControl)!= 0) ShowObject(LearnWrongPushButton);
	else 	HideObject(LearnWrongPushButton);

	//bingo push
	if (GetBitMacro(controlFlag.allBits, bingoControl)!= 0) ShowObject(LearnBingoPushButton);
	else	HideObject(LearnBingoPushButton);

	//pass push
	if (GetBitMacro(controlFlag.allBits, passControl)!= 0)
		FrmShowObject(frmP, FrmGetObjectIndex(frmP, LearnPassPushButton));
	else
		FrmHideObject(frmP, FrmGetObjectIndex(frmP, LearnPassPushButton));
	
	//fail push
	if (GetBitMacro(controlFlag.allBits, failControl)!= 0)
		FrmShowObject(frmP, FrmGetObjectIndex(frmP, LearnFailPushButton));
	else
		FrmHideObject(frmP, FrmGetObjectIndex(frmP, LearnFailPushButton));
	
	if (PushSelection == noSelectedControl){
	
		if (controlFlag.allBits == ShowRightWrong){
			SwitchButtonOff(LearnRightPushButton);		
			SwitchButtonOff(LearnWrongPushButton);
		}
	
		if (controlFlag.allBits == ShowBingoPassFail){
			SwitchButtonOff(LearnBingoPushButton);
			SwitchButtonOff(LearnPassPushButton);
			SwitchButtonOff(LearnFailPushButton);
		}
	}
	
	
	if (PushSelection == rightControl) 
		FrmSetControlGroupSelection(frmP, RightWrongGroup, LearnRightPushButton);
	
	if (PushSelection == wrongControl)
		FrmSetControlGroupSelection(frmP, RightWrongGroup, LearnWrongPushButton);
		
	if (PushSelection == bingoControl)
		FrmSetControlGroupSelection(frmP, BingoPassFailGroup, LearnBingoPushButton);
	
	if (PushSelection == passControl)
		FrmSetControlGroupSelection(frmP, BingoPassFailGroup, LearnPassPushButton);
		
	if (PushSelection == failControl)
		FrmSetControlGroupSelection(frmP, BingoPassFailGroup, LearnFailPushButton);

}

Boolean LoadWord(){
	
	HeaderItem item;
	LearningStatus status;
	Err error;
	
	while(true) {
		if (0 == GetHeaderItemNumber(CurrentDB, HeaderRecordIndex)){
			Finish();
			return false;
		}
		
		GetHeaderItem(CurrentDB, HeaderRecordIndex, &item, 0); //fetch first item
		error = DmFindRecordByID(CurrentDB, item, &CurrentRecord);
		if (error) ErrAlert(error);
		if (error == errNone) break; 
		RemoveHeaderItem(CurrentDB, HeaderRecordIndex, 0);
	}
	
	if (HeaderRecordIndex == LearningRecordIndex){
		status = GetLearningStatus(CurrentDB, CurrentRecord);
				
		switch(status){
			case FirstLook:
				ShowAnswerPane = true;
				controlFlag.allBits = ShowNext;
				fieldFlag.allBits = GetAnswerPageFields();
				break;
			
			case TranslationRecollection:
			case WordRecollection:
			case TranslationTest:
			case WordTest:
				ShowAnswerPane = false;
				controlFlag.allBits = ShowShowAnswer;
				fieldFlag.allBits = GetQuestionPageFields();
				break;
		}				
	}
	else if (HeaderRecordIndex == TestingRecordIndex){
		ShowAnswerPane = false;
		controlFlag.allBits = ShowShowAnswer;
		fieldFlag.allBits = GetQuestionPageFields();
	}
	
	CurrentTop = GetCanvasBounds().topLeft.y;
	
	return true;
}

UInt16 GetQuestionPageFields(){
	WordFieldFlags flag;
	SugarAppInfoType* app;
	app = GetAppInfoPtr(CurrentDB);
	flag = app->questionPageFields;
	MemPtrUnlock(app);
	return flag.allBits;	
}

UInt16 GetAnswerPageFields(){
	WordFieldFlags flag;
	SugarAppInfoType* app;
	app = GetAppInfoPtr(CurrentDB);
	flag = app->answerPageFields;
	MemPtrUnlock(app);
	return flag.allBits;	
}


void DoControlShowButton(){

	ShowAnswerPane = true;
	if (HeaderRecordIndex == LearningRecordIndex){
		

		if (GetLearningStatus(CurrentDB, CurrentRecord) == TranslationTest){
			controlFlag.allBits = ShowRightWrong;
		}
	
		else{
			
			controlFlag.allBits = ShowNext;
		}
		
	}
	
	if (HeaderRecordIndex == TestingRecordIndex){
		controlFlag.allBits = ShowBingoPassFail;
	}
	
	fieldFlag.allBits = GetAnswerPageFields();

	CurrentTop = GetCanvasBounds().topLeft.y;
	PushSelection = noSelectedControl;
	LoadControls();
	DrawCanvas();
}

void DoControlNext(){
/*
	HeaderQueue queue;
	LearningStatus learningStatus;
	TestingStatus testingStatus;
	MemorizingStatus memoStatus;
	DateType zeroDate = {0, 0, 0};
	DateType today;
	Int32 interval;
	FormType* frmP;
	ControlType* ctl;
	
	ClearAnswerTab();
	
	DateSecondsToDate(TimGetSeconds(), &today);
	
	if (HeaderRecordIndex == LearningRecordIndex){
		queue = GetQueue(CurrentDB, CurrentRecord);
		learningStatus = GetLearningStatus(CurrentDB, CurrentRecord);
		memoStatus = GetMemorizingStatus(CurrentDB, CurrentRecord);
		
		switch(learningStatus){
			case FirstLook:
				SetLearningStatus(CurrentDB, CurrentRecord, TranslationRecollection);
				MoveHeaderItem(CurrentDB, HeaderRecordIndex, 6);
				break;		
			
			case TranslationRecollection:
				SetLearningStatus(CurrentDB, CurrentRecord, WordRecollection);
				MoveHeaderItem(CurrentDB, HeaderRecordIndex, 13);
				break;
			
			case WordRecollection:
				SetLearningStatus(CurrentDB, CurrentRecord, TranslationTest);
				MoveHeaderItem(CurrentDB, HeaderRecordIndex, 55);
				break;
				
			case TranslationTest:

				if (PushSelection == rightControl){ //right button pushed
					SetQueue(CurrentDB, CurrentRecord, TestingQueue);
					
					//clear learning/testing status
					SetLearningStatus(CurrentDB, CurrentRecord, FirstLook);     //i.e. clear ltstatus
					
					//set memo status
					memoStatus.difficulty = Difficulty(memoStatus.eFactor);
					memoStatus.step = 1;
					memoStatus.currentTest = zeroDate;
					memoStatus.nextTest = today;
					
					interval = 8 - (Int32)((memoStatus.eFactor - 1.3) * 3 / 1.2);
					
					DateAdjust(&memoStatus.nextTest, interval);

					//set first learn;
					if (ZeroDate(memoStatus.firstLearn))
						memoStatus.firstLearn = today;
										
					//set memo status
					SetMemorizingStatus(CurrentDB, CurrentRecord, memoStatus);	
				}
				else if (PushSelection == wrongControl){
					MoveHeaderItem(CurrentDB, HeaderRecordIndex, 55);
				}
				
				//!!! Clear Selection
				frmP = FrmGetActiveForm();
				ctl = FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, LearnWrongPushButton));
				CtlSetValue(ctl, 0);
				ctl = FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, LearnRightPushButton));
				CtlSetValue(ctl, 0);

				PushSelection = noSelectedControl;
				break;
		
		}//end switching learning status
	
	} //end if learning
	
	else if (HeaderRecordIndex == TestingRecordIndex){
		
		queue = GetQueue(CurrentDB, CurrentRecord); 
		testingStatus = GetTestingStatus(CurrentDB, CurrentRecord);
		memoStatus = GetMemorizingStatus(CurrentDB, CurrentRecord);
		
		
		if (queue == TestingQueue){
			switch (PushSelection){
				
				case bingoControl:
					
					if (memoStatus.step == 1){
						//bingo times accumulate
						testingStatus.bingoed = 1;	
						testingStatus.passed = 0;
						testingStatus.failed = 0;
						
						
						memoStatus.totalRecall += 1;
						memoStatus.step = 2;
						memoStatus.lastTest = today;
						memoStatus.currentTest = zeroDate;
						memoStatus.nextTest = today;
						interval = (Int32)((memoStatus.eFactor - 1.3) * 8 / 1.2) + 13;
						DateAdjust(& memoStatus.nextTest, interval); 
						break;
					}
					
					//if successive bingo, adjust eFactor
					if (testingStatus.bingoed > 2){			
						testingStatus.bingoed = 0;
						testingStatus.passed = 0;
						testingStatus.failed = 0;
						
						memoStatus.eFactor += 0.1;
						memoStatus.difficulty = Difficulty(memoStatus.eFactor);
						
						interval = (DateToDays(memoStatus.nextTest) - DateToDays(memoStatus.lastTest)) * (memoStatus.eFactor - 0.1);
						memoStatus.totalRecall += 1;
						memoStatus.step += 1;
						memoStatus.lastTest = today;
						memoStatus.currentTest = zeroDate;
						memoStatus.nextTest = today;
						DateAdjust(& memoStatus.nextTest, interval);
						break;	
					}
					// do not adjust efactor					
					
					testingStatus.bingoed += 1;
					testingStatus.passed = 0;
					testingStatus.failed = 0;

					interval = (DateToDays(memoStatus.nextTest) - DateToDays(memoStatus.lastTest)) * (memoStatus.eFactor - 0.1);
					memoStatus.totalRecall += 1;
					memoStatus.step += 1;
					memoStatus.lastTest = today;
					memoStatus.currentTest = zeroDate;
					memoStatus.nextTest = today;
					DateAdjust(& memoStatus.nextTest, interval);
					
					break;
									
				case passControl:
					//queue = PassedQueue;
					testingStatus.bingoed = 0;
					testingStatus.passed = 1;
					testingStatus.failed = 0;
					memoStatus.currentTest = today;
					
					break;
					
				case failControl:
					//queue = FailedQueue;
					testingStatus.bingoed = 0;
					testingStatus.passed = 0;
					testingStatus.failed = 1;

					break;						
			
			}
		
		}
		
		else if (queue == TestingQueue){
			switch (PushSelection){
				case bingoControl:
					if (memoStatus.step == 1){
						queue = TestingQueue;
						//bingo times accumulate
						testingStatus.bingoed = 0;	
						testingStatus.passed = 0;
						testingStatus.failed = 0;
						
						memoStatus.eFactor -= 0.14;
						if (memoStatus.eFactor < 1.3) memoStatus.eFactor = 1.3;
						
						memoStatus.difficulty = Difficulty(memoStatus.eFactor);
						memoStatus.totalRecall += 1;
						memoStatus.step = 2;
						memoStatus.lastTest = today;
						memoStatus.currentTest = zeroDate;
						memoStatus.nextTest = today;
						
						interval = (Int32)((memoStatus.eFactor - 1.3) * 8 / 1.2) + 13;
						DateAdjust(& memoStatus.nextTest, interval); 
						break;
					}
					
					
					
					queue = TestingQueue;
					
					testingStatus.bingoed = 0;
					testingStatus.passed = 0;
					testingStatus.failed = 0;
					
					memoStatus.eFactor -= 0.14;
					if (memoStatus.eFactor < 1.3) memoStatus.eFactor = 1.3;
					
					memoStatus.difficulty = Difficulty(memoStatus.eFactor);
					memoStatus.totalRecall += 1;
					memoStatus.step += 1;
					
					interval = (DateToDays(memoStatus.nextTest) - DateToDays(memoStatus.lastTest)) * (memoStatus.eFactor - 0.1);
					memoStatus.lastTest = today;
					memoStatus.currentTest = zeroDate;
					memoStatus.nextTest = today;
					DateAdjust(&memoStatus.nextTest, interval);

					break;
					
				case passControl:
					testingStatus.passed +=1;
					break;
			
				case failControl:
					//queue = FailedQueue;
					testingStatus.bingoed = 0;
					testingStatus.passed = 0;
					testingStatus.failed = 1;

					break;
			}
		
		}
		else if (queue == TestingQueue){
			switch(PushSelection){
				case bingoControl:
					if (testingStatus.failed == 0){
						queue = TestingQueue;
						
						testingStatus.bingoed = 0;
						testingStatus.passed = 0;
						testingStatus.failed = 0;
						
						memoStatus.eFactor -= 0.32;
						if (memoStatus.eFactor < 1.3) memoStatus.eFactor = 1.3;
						
						memoStatus.difficulty = Difficulty(memoStatus.eFactor);
						memoStatus.totalLapse += 1;
						memoStatus.step = 1;
						memoStatus.lastTest = today;
						memoStatus.currentTest = zeroDate;
						memoStatus.nextTest = today;
						
						interval = 8 - (Int32)((memoStatus.eFactor - 1.3) * 3 / 1.2);
						DateAdjust(&memoStatus.nextTest, interval);
						break;
					}
					
					if ((testingStatus.bingoed + testingStatus.passed) > testingStatus.failed){
						queue = TestingQueue;
						
						testingStatus.bingoed = 0;
						testingStatus.passed = 0;
						testingStatus.failed = 0;
						
						memoStatus.eFactor -= 0.54;
						if (memoStatus.eFactor < 1.3) memoStatus.eFactor = 1.3;
						
						memoStatus.difficulty = Difficulty(memoStatus.eFactor);
						memoStatus.totalLapse += 1;
						memoStatus.step = 1;
						memoStatus.lastTest = today;
						memoStatus.currentTest = zeroDate;
						memoStatus.nextTest = today;
						
						interval = 8 - (Int32)((memoStatus.eFactor - 1.3) * 3 / 1.2);
						DateAdjust(&memoStatus.nextTest, interval);
						break;
					}
					
									
					testingStatus.bingoed += 1;
					
					break;
					
				case passControl:
					testingStatus.passed += 1;
					break;
					
				case failControl:
					if (testingStatus.failed > 3){
						queue = LearningQueue;
						
						testingStatus.bingoed = 0;
						testingStatus.passed = 0;
						testingStatus.failed = 0;
						
						memoStatus.eFactor -= 0.8;
						if (memoStatus.eFactor < 1.3) memoStatus.eFactor = 1.3;
						
						memoStatus.difficulty = Difficulty(memoStatus.eFactor);
						memoStatus.totalLapse += 1;
						memoStatus.step = 0;
						memoStatus.lastTest = today;
						memoStatus.currentTest = zeroDate;
						memoStatus.nextTest = zeroDate;						
					}
					else {
						testingStatus.failed += 1;
					}
					break;
					
			}	//end switch control
		}//end if failed queue
		
		SetTestingStatus(CurrentDB, CurrentRecord, testingStatus);
		SetMemorizingStatus(CurrentDB, CurrentRecord, memoStatus);
		SetQueue(CurrentDB, CurrentRecord, queue);
		
		//if (queue == TestingQueue || queue == LearningQueue)
		//	RemoveHeaderItem(CurrentDB, HeaderRecordIndex, 0);
		
		//if (queue == LearningQueue){
		//	DmRecordInfo(CurrentDB, CurrentRecord, NULL, &item, NULL);
		//	AddHeaderItem(CurrentDB, LearningRecordIndex, &item);
		//}
		
		if (queue == TestingQueue){
			interval = TimGetSeconds()%40;
			MoveHeaderItem(CurrentDB, HeaderRecordIndex, 40 + (UInt16)interval);
		}
		
		//clear push selection
		PushSelection = noSelectedControl;

	}//end if testing
	
	if (LoadWord()){
		LoadControls();
		DrawBanner();
		FrmDrawForm(FrmGetActiveForm());
		DrawCanvas();
	}*/
}

Boolean LearnDoHardKey(EventType* event){
	Boolean rightKeyPushed = false;
	Boolean leftKeyPushed = false;
	Boolean prevField = false;
	Boolean nextField = false;
	FormType* frmP = FrmGetActiveForm();
	UInt16 controlID[WordFieldsCount];
	UInt16 i, j, k, result, objectID;
	SugarAppInfoType* app;
	WordRecordType record;
	MemHandle recordH;

	switch(event ->data.keyDown.chr){
		case hard1Chr:
			if (Lefty) leftKeyPushed = true;
			else prevField = true;
			break;
			
		case hard3Chr:
			if (Lefty)	prevField = true;
			else leftKeyPushed = true;
			break;
			
		case hard2Chr:
			if (Lefty)	rightKeyPushed = true;
			else  nextField = true;
			break;
			
		case hard4Chr:
			if (Lefty) nextField = true;
			else rightKeyPushed = true;
			break;
			
		default:
			break;
	
	}
	
	if (rightKeyPushed){
		SndPlaySystemSound(sndClick);
	
		if (GetBitMacro(controlFlag.allBits, nextControl)) {
			DoControlNext();
			return true;;
		}
		
		if (GetBitMacro(controlFlag.allBits, showControl)) {
			DoControlShowButton();
			return true;
		}
		
		if (GetBitMacro(controlFlag.allBits, bingoControl) || GetBitMacro(controlFlag.allBits, rightControl)){
			if (PushSelection != noSelectedControl){
				DoControlNext();
			}
		}
		
		return true;	
	}
	
	if (leftKeyPushed){
		SndPlaySystemSound(sndClick);
		
		if (GetBitMacro(controlFlag.allBits, rightControl)) {
			
			if (PushSelection == rightControl)	PushSelection = wrongControl;
			else PushSelection = rightControl;
			
			controlFlag.allBits = ShowRightWrong;
			
			LoadControls();
			return true;
		}
		
		if (GetBitMacro(controlFlag.allBits, bingoControl)){
		
			if (PushSelection == bingoControl)	PushSelection = passControl;
			else if (PushSelection == passControl)	PushSelection = failControl;
			else PushSelection = bingoControl;
			
			controlFlag.allBits = ShowBingoPassFail;
			LoadControls();
			return true;
		
		}
		
		return true;
	}
	
	if (prevField || nextField){
		SndPlaySystemSound(sndClick);
	
		if (GetBitMacro(controlFlag.allBits, showControl)) return true;
		
		objectID = (UInt16)FrmGetControlGroupSelection(FrmGetActiveForm(), DynamicPushButtonGroup);
		
		if (objectID == frmNoSelectedControl) return true;
		
		app = GetAppInfoPtr(CurrentDB);
		GetWordRecord(CurrentDB, CurrentRecord, &record, &recordH);
		j = 0;
		
		for (i = WordFieldQuestion; i < WordFieldsCount; i++){
			
			if (i == DefaultAnswerField	|| (GetBitMacro(app -> answerPageFields.allBits, i) != 0) 
				&& (record.field[i] != NULL)){
			
				controlID[j] = DynamicPushButtonStart + i;
				j++;
				
			}
		
		}
		MemPtrUnlock(app);
		MemHandleUnlock(recordH);
		
		for (k = 0; k < j; k++){
			if (FrmGetObjectIndex(FrmGetActiveForm(), controlID[k]) == objectID) {
				break;
			}
		}
		
		if (k == j) return true;
		if (prevField){
			if (k == 0)	result = controlID[j-1];
			else result = controlID[k - 1];
		}
		else {
			if (k == j - 1) result = controlID[0];
			else result = controlID[k + 1];
		}
		
		FrmSetControlGroupSelection(FrmGetActiveForm(), DynamicPushButtonGroup, result);
		CurrentAnswerField = result - DynamicPushButtonStart;
		CurrentAnswerTop = 0;
		DrawAnswerPane();

		return true;
	}
	
	return true;

}


UInt8 Difficulty(float efactor){

	UInt8 x = (UInt8)((efactor - 1.3)*50);
	if (x > 100) return (UInt8) 100;
	return (UInt8)(100 - x);

}


RectangleType GetCanvasBounds(){	

	RectangleType bounds;
	FormType* frmP = FrmGetActiveForm();
	UInt16 index = FrmGetObjectIndex(frmP, LearnCanvasGadget);

	FrmGetObjectBounds(frmP, index, &bounds);

	return bounds;
}

void Finish(){

//	FrmCustomAlert(FinishedAlert, "learn", NULL, NULL);
//	if (CurrentDB != NULL) DmCloseDatabase(CurrentDB);
// 	FrmGotoForm(MainForm);

}

void SwitchButtonOff(UInt16 controlID){

	ControlType* ctl = GetObjectPtr(controlID);
	CtlSetValue(ctl, 0);
	CtlDrawControl(ctl);

}

void SwitchButtonOn(UInt16 controlID){

	ControlType* ctl = GetObjectPtr(controlID);
	CtlSetValue(ctl, 1);
	CtlDrawControl(ctl);

}

void DrawBanner(){
	
	
	FormType * frmP = FrmGetActiveForm();
	RectangleType bounds;
	Char p[20];
	FontID currFont;
	LearningStatus s;
	MemorizingStatus ms;
	UInt16 i;
	Char lamp[5];
	
	
	if (ShowAnswerPane == true) return;
	if (HeaderRecordIndex == LearningRecordIndex){
		s = GetLearningStatus(CurrentDB, CurrentRecord);
	
		switch(s){
			case FirstLook:
				StrPrintF(p, "%s", "1st look");
				break;
			
			case TranslationRecollection:
				StrPrintF(p, "%s", "recall");
				break;
			
			case WordRecollection:
				StrPrintF(p, "%s", "2nd recall");
				break;
			
			case TranslationTest:
				StrPrintF(p, "%s", "final");
				break;
		
			default:
				break;
		}
	
	}
	
	if (HeaderRecordIndex == TestingRecordIndex){
		ms = GetMemorizingStatus(CurrentDB, CurrentRecord);
		for (i = 0; i < 5; i++){
			if (ms.difficulty <= i * 20) lamp[i] = 'H';
			else if (ms.difficulty - i * 20 <= 10) lamp[i] = 'G';
			else lamp[i] = 'F';
		}
		
		StrPrintF(p, "%c%c%c%c%c", lamp[0], lamp[1], lamp[2], lamp[3], lamp[4]);
	
	}

	
	FrmGetObjectBounds(frmP, GetObjectIndex(LearnTitleBannerGadget),&bounds);
		
	WinEraseRectangle(&bounds, 0);
	
	if (HeaderRecordIndex == LearningRecordIndex) currFont = FntSetFont(boldFont);
	else currFont = FntSetFont(PhoneticSmallFont);
	
	WinPushDrawState();
	WinSetTextColor(UIColorGetTableEntryIndex(UIObjectSelectedFill));

	WinDrawChars(p, StrLen(p), bounds.topLeft.x, bounds.topLeft.y);
	WinPopDrawState();
	
	FntSetFont(currFont);

}

void LoadAnswerTab(){
	
	FormType* frmP;
	FormType** formPP;
	SugarAppInfoType* app;
	WordRecordType record;
	MemHandle recordH;
	UInt16 xpos = 1, ypos = 147, width = 12, height = 12;
	WordFields i;	
	
	app = GetAppInfoPtr(CurrentDB);
	GetWordRecord(CurrentDB, CurrentRecord, &record, &recordH);
	
	for (i = WordFieldQuestion; i < WordFieldsCount; i++){
		
		if (i == DefaultAnswerField	|| (GetBitMacro(app -> answerPageFields.allBits, i) != 0) 
			&& (record.field[i] != NULL)){
			
			frmP = FrmGetActiveForm();
			formPP = &frmP;
			fieldTab[i] = CtlNewControl((void**)formPP, DynamicPushButtonStart + i, pushButtonCtl, 
				fieldLabel[i], xpos, ypos, width, height, stdFont, DynamicPushButtonGroup, false);
			
			CtlDrawControl(fieldTab[i]);
			xpos += width + 1;			
		}
	
	}
	MemPtrUnlock(app);
	MemHandleUnlock(recordH);
	
	FrmSetControlGroupSelection(FrmGetActiveForm(), DynamicPushButtonGroup, DynamicPushButtonStart + DefaultAnswerField);
}

void ClearAnswerTab(){
	
	FormType* frmP;
	FormType** formPP;
	
	UInt16 index;
	WordFields i;
	
	SugarAppInfoType* app;
	WordRecordType record;
	MemHandle recordH;
	
	app = GetAppInfoPtr(CurrentDB);
	GetWordRecord(CurrentDB, CurrentRecord, &record, &recordH);
	
	for (i = WordFieldQuestion; i < WordFieldsCount; i++){
		if (i == DefaultAnswerField	|| (GetBitMacro(app -> answerPageFields.allBits, i) != 0) 
			&& (record.field[i] != NULL)){
		
			frmP = FrmGetActiveForm();
			index = FrmGetObjectIndex(frmP, DynamicPushButtonStart + i);
		
			if (index == frmInvalidObjectId) continue;
		
			formPP = &frmP;
		
			FrmRemoveObject(formPP, index);
		}
	
	}
	
	MemPtrUnlock(app);
	MemHandleUnlock(recordH);
	
	WinEraseRectangle(&ToolBar, 0);
}
