#include <PalmOS.h>

#include "SugarMemo.h"
#include "SugarMemoRsc.h"
#include "SugarMemoLearn2.h"
#include "SugarMemoDB.h"
#include "SugarMemoTools.h"

//	Sony Jog
#define 	vchrJogUp				(0x1700)
#define 	vchrJogDown				(0x1701)
#define 	vchrJogPushRepeat		(0x1702)
#define 	vchrJogPushedUp			(0x1703)
#define 	vchrJogPushedDown		(0x1704)
#define 	vchrJogPush				(0x1705)
#define 	vchrJogRelease			(0x1706)
#define 	vchrJogBack				(0x1707) 		/* added @ 2001 */


#define 	RowsInChoiceTable 	(Int16)7
#define 	ChoiceTableRowHeight		15

#define 	recollectionMode	true
#define 	testMode			false

#define 	BingoPassFailGroup 		17
#define		RightWrongGroup			23
#define 	DynamicPushButtonGroup 	6666
#define 	DynamicPushButtonStart	1234

#define		maxFail					8

WordFieldFlags UpperPaneFlag;
WordFieldFlags LowerPaneFlag;

UInt16 		UpperPaneTopAnchor;
UInt16 		UpperPaneFirstLineHeight;
Boolean 	UpperPaneScrollableUp;
Boolean		UpperPaneScrollableDown;
UInt16 		LowerPaneTopAnchor;
UInt16 		LowerPaneFirstLineHeight;
Boolean		LowerPaneScrollableUp;
Boolean		LowerPaneScrollableDown;

Boolean 	ShowNext = false;
Boolean 	ShowShow = false;
Boolean 	ShowChoices = false;
Boolean 	ShowRightWrong = false;
Boolean 	ShowBingoPassFail = false;

UInt32		StartingTicks;

WordFields	DefaultField;
WordFields	ChoiceField;

LearningStatus	CurrentLearningStatus;
ControlType* 	FieldTab[WordFieldsCount];
extern EditLabel fieldLabel[WordFieldsCount];

static Boolean 	Learn2FormDoCommand(UInt16 command);
static void 	Learn2FormInitChoiceBuffer();
static void 	Learn2FormInitChoiceTable();
static void		Learn2InitWord();
static void 	Learn2FormQuit();
static void 	DrawChoice(void* table, Int16 row, Int16 column, RectangleType* bounds);
static void 	StrPackString(char* P);
static void 	RemoveBlankHead(char** p);
static Boolean 	DrawField(RectangleType* bounds, char* p, UInt16* top, UInt16 align, 
	UInt16* topAnchor, UInt16* firstLineHeight, Boolean draw);
static Boolean 	DrawFields(RectangleType* bounds, WordFieldFlags flag, UInt16* top, UInt16* topAnchor, 
	UInt16* firstLineHeight, Boolean draw);
static Boolean 	DrawPane(RectangleType* bounds, WordFieldFlags flag, UInt16* topAnchor, UInt16* firstLineHeight);
static void 	DrawUpperPane(WordFieldFlags flag);
static void 	DrawLowerPane(WordFieldFlags flag);
static void 	LoadLowerPaneTab();
static void		ClearLowerPaneTab();
static void 	DrawChoiceTable();
static void		ResetUpperPane();
static void 	ResetLowerPane();
static void		ResetAllPush();
static void		Learn2FormDoButtonNext();
static void 	Learn2FormDoButtonShow();
static void		Learn2FormDoButtonPush(UInt16 controlID);
static void 	Learn2FormTableSelection(Int16 row);
static void		ClearChoiceTable();
static void 	ErasePane(UInt16 ObjectID);
static void 	Learn2ScrollUpperPane(Boolean up);
static void 	Learn2ScrollLowerPane(Boolean up);

static LearningStatus TossNextReviewMode();
static UInt8 Difficulty(float efactor);
static void 	Learn2ProcessReviewRating(Rating rating);
static void		Learn2ProcessLearningRating(Boolean right);
static void		Learn2FinishLearning();
static void 	Learn2FinishReview();
static void 	DrawTranslationLook();
static void 	DrawWordLook();
static void 	DrawTranslationRecollection();
static void 	DrawWordRecollection();
static void		DrawTranslationTest();
static void 	DrawWordTest();

static void		Learn2FormInit();

static Boolean 	Lefty();
static WordFieldFlags 	GetQuestionFields();
static WordFieldFlags 	GetAnswerFields();

static Boolean	LoadWord();
static void 	Learn2DoDynamicPushButton(WordFields field);
static void 	ToggleDynamicField(Boolean prevField);
static void 	Learn2ScrollChoiceSelection(Boolean up);

static void		Learn2ClipBoardWord();

static UInt16 	FontIDToListIndex(FontID font);
static FontID 	ListIndexToFontID(Int16 index);

Boolean Learn2FormHandleEvent(EventType * event){

	FormType* frmP = FrmGetActiveForm();
	Boolean handled = false;
	
	RectangleType bounds;
			
	switch (event->eType){
		
		case frmOpenEvent:
			
			Learn2FormInit();
			FrmDrawForm(frmP);
			if (LoadWord()) Learn2InitWord();
			else Learn2FormQuit();
			handled = true;
			break;
			
		case menuEvent:
			return Learn2FormDoCommand(event->data.menu.itemID);

		case penDownEvent:
			GetObjectBounds(Learn2UpperPaneScrollUpRepeating, &bounds);
			if (UpperPaneScrollableUp && RctPtInRectangle(event->screenX, event->screenY, &bounds)) 
				return false;
				
			GetObjectBounds(Learn2UpperPaneScrollDownRepeating, &bounds);
			if (UpperPaneScrollableDown && RctPtInRectangle(event->screenX, event->screenY, &bounds)) 
				return false;
			
			GetObjectBounds(Learn2UpperPaneGadget, &bounds);
			if (RctPtInRectangle(event->screenX, event->screenY, &bounds)){
				SndPlaySystemSound(sndClick);
				FrmGotoForm(EditForm);
				handled = true;
			}
			break;
			
		case ctlSelectEvent:
					
			if (event->data.ctlSelect.controlID - DynamicPushButtonStart < WordFieldsCount){
				Learn2DoDynamicPushButton(event->data.ctlSelect.controlID - DynamicPushButtonStart);
				handled = true;	
			}
			else {
				switch(event->data.ctlSelect.controlID){
					case Learn2NextButton:
						Learn2FormDoButtonNext();
						handled = true;
						break;
				
					case Learn2ShowButton:
						Learn2FormDoButtonShow();
						handled = true;
						break;
					
					case Learn2RightPushButton:
					case Learn2WrongPushButton:
					case Learn2BingoPushButton:
					case Learn2PassPushButton:
					case Learn2FailPushButton:
						Learn2FormDoButtonPush(event->data.ctlSelect.controlID);
						handled = true;
						break;
			
				}
			}
			break;
			
		case ctlRepeatEvent:
			switch(event->data.ctlRepeat.controlID){
				case Learn2UpperPaneScrollUpRepeating:
					Learn2ScrollUpperPane(true);
					break;
				case Learn2UpperPaneScrollDownRepeating:
					Learn2ScrollUpperPane(false);
					break;
				case Learn2LowerPaneScrollUpRepeating:
					Learn2ScrollLowerPane(true);
					break;
				case Learn2LowerPaneScrollDownRepeating:
					Learn2ScrollLowerPane(false);
					break;
			}
			break;
		
		case tblSelectEvent:
			Learn2FormTableSelection(event->data.tblSelect.row);
			handled = true;
			break;
			
		case keyDownEvent:
			if (EvtKeydownIsVirtual(event))
			{
				switch (event->data.keyDown.chr)
				{
					case vchrPageUp:
						if (ShowChoices){
							Learn2ScrollChoiceSelection(true);
						}
						else if (LowerPaneScrollableUp){
							Learn2ScrollLowerPane(true);
						}
						
						handled = true;
						break;
		
					case vchrPageDown:
						if (ShowChoices){
							Learn2ScrollChoiceSelection(false);
						}
						else if (LowerPaneScrollableDown){
							Learn2ScrollLowerPane(false);
						}
						handled = true;
						break;
						
					case	vchrJogUp:
					case	vchrJogDown:
					case	vchrJogPushRepeat:
					case	vchrJogPushedUp:
					case	vchrJogPushedDown:
					case	vchrJogPush:
					case	vchrJogRelease:
					case 	vchrJogBack:
						 return Learn2FormDoHardKey(event);
					
					default:
						break;
				}
			}
			break;
		
			
		default:
			break;
	}
	return handled;			
}

void Learn2FormInit(){
	
	//bingo, pass, fail push
	//right, wrong, push
	//next, show button
	//lowerpane scroll repeating
	FormType* frmP = FrmGetActiveForm();
	
	if (Lefty()){

		FrmSetObjectPosition(frmP, GetObjectIndex(Learn2BingoPushButton), 47, 145); 	//bingo push
		FrmSetObjectPosition(frmP, GetObjectIndex(Learn2PassPushButton), 23, 145);
		FrmSetObjectPosition(frmP, GetObjectIndex(Learn2FailPushButton), 1, 145);
	
		FrmSetObjectPosition(frmP, GetObjectIndex(Learn2RightPushButton), 38, 145);
		FrmSetObjectPosition(frmP, GetObjectIndex(Learn2WrongPushButton), 1, 145);

		FrmSetObjectPosition(frmP, GetObjectIndex(Learn2NextButton), 1, 145);
		FrmSetObjectPosition(frmP, GetObjectIndex(Learn2ShowButton), 1, 145);

		FrmSetObjectPosition(frmP, GetObjectIndex(Learn2LowerPaneScrollUpRepeating), 88, 145);
		FrmSetObjectPosition(frmP, GetObjectIndex(Learn2LowerPaneScrollDownRepeating), 88, 153);
		
	}
	else {
		
		FrmSetObjectPosition(frmP, GetObjectIndex(Learn2BingoPushButton), 86, 145); 	//bingo push
		FrmSetObjectPosition(frmP, GetObjectIndex(Learn2PassPushButton), 114, 145);
		FrmSetObjectPosition(frmP, GetObjectIndex(Learn2FailPushButton), 138, 145);
	
		FrmSetObjectPosition(frmP, GetObjectIndex(Learn2RightPushButton), 86, 145);
		FrmSetObjectPosition(frmP, GetObjectIndex(Learn2WrongPushButton), 123, 145);

		FrmSetObjectPosition(frmP, GetObjectIndex(Learn2NextButton), 86, 145);
		FrmSetObjectPosition(frmP, GetObjectIndex(Learn2ShowButton), 86, 145);

		FrmSetObjectPosition(frmP, GetObjectIndex(Learn2LowerPaneScrollUpRepeating), 60, 145);
		FrmSetObjectPosition(frmP, GetObjectIndex(Learn2LowerPaneScrollDownRepeating), 60, 153);
	
	}

	ShowNext = false;
 	ShowShow = false;
	ShowChoices = false;
	ShowRightWrong = false;
 	ShowBingoPassFail = false;

	HideObject(Learn2LowerPaneGadget);
	HideObject(Learn2UpperPaneGadget);
	Learn2FormInitChoiceTable();
	Learn2FormInitChoiceBuffer();
}

Boolean Learn2FormDoCommand(UInt16 command){
	
	Boolean handled = false;
	
	switch(command){
		
		case LearnOptionsMenuItem1300:  // font settings
			FrmGotoForm(FontForm);
			handled = true;
			break;
		
		case LearnOptionsMenuItem1302:	// quit
			Learn2FormQuit();
			handled = true;
			break;
			
		default:
			break;
	}
	return handled;
}

void Learn2FormInitChoiceTable(){
	
	Int16 row;
	
	TableType* table = GetObjectPtr(Learn2ChoiceTable);
	
	for (row = 0; row < RowsInChoiceTable; row++){
		TblSetItemStyle(table, row, 0, customTableItem);	
		TblSetRowHeight(table, row, 16);
		TblSetItemFont(table, row, 0, largeBoldFont);
		TblSetRowUsable(table, row, false);
	}
	
	TblSetColumnUsable(table, 0, true);
	TblSetCustomDrawProcedure(table, 0, DrawChoice);
}

void Learn2FormInitChoiceBuffer(){
	
	switch(HeaderRecordIndex){
		case LearningRecordIndex:
			InitHeaderBuffer(CurrentDB, LearningChoiceBuffer);
			break;
			
		case TestingRecordIndex:
			InitHeaderBuffer(CurrentDB, TestingChoiceBuffer);
			break;
			
		default:
			ErrDisplay("Error HeaderRecordIndex in Learn2FormInitChoiceBuffer");
			break;
	}
}

void DrawChoice(void* table, Int16 row, Int16 column, RectangleType* bounds){
	
	WordRecordType record;
	MemHandle recordH;
	
	char* strP;
	MemHandle strH;
	
	UInt16 index = TblGetRowID(table, row);
	Int16 strWidth, strLength;
	Boolean fit;
	FontID currFont;
	
	if (index == noRecord) return;
	
	GetWordRecord(CurrentDB, index, &record, &recordH);
	
	if (record.field[ChoiceField] == NULL || StrLen(record.field[ChoiceField]) == 0){
		MemHandleUnlock(recordH);
		return;
	}
	
	strH = MemHandleNew(StrLen(record.field[ChoiceField]) + 1);
	if (strH == NULL) return;
	
	strP = MemHandleLock(strH);
	
	StrCopy(strP, record.field[ChoiceField]);
	MemHandleUnlock(recordH);

//	Omitted by hytown to avoid phrase messy
//	StrPackString(strP);
		
	currFont = FntSetFont(largeBoldFont);
	strWidth = bounds->extent.x;
	strLength = StrLen(strP);
	FntCharsInWidth(strP, &strWidth, &strLength, &fit);
		
	if (!fit){
		FntSetFont(boldFont);
		strWidth = bounds->extent.x;
		strLength = StrLen(strP);
		FntCharsInWidth(strP, &strWidth, &strLength, &fit);
			
		if (!fit){
			strWidth = bounds->extent.x;
			strLength = StrLen(strP);
			FntSetFont(stdFont);
			FntCharsInWidth(strP, &strWidth, &strLength, &fit);
		}
	}
		
	WinDrawChars(strP, StrLen(strP), bounds->topLeft.x + (bounds->extent.x - strWidth)/2, 
		bounds -> topLeft.y + (bounds -> extent.y + 2 - FntLineHeight())/2 - 1);
			
	MemHandleFree(strH);
	FntSetFont(currFont);

}

void StrPackString(char* P){
	char* P1, *p, *p1;
	MemHandle h;
	
	if (P == NULL || StrLen(P) == 0) return;
	h = MemHandleNew(StrLen(P) + 1);
	if (h == NULL) return;
	P1 = MemHandleLock(h);
	
	p = P;
	p1 = P1;
	
	while (true){

		if ((*p) != 9 && (*p) != 10 && (*p) != 13 && (*p) != 32){
			(*p1) = (*p);
			if ((*p) == '\0') break;
			p1 ++;
		}
		p++;
	}

	StrCopy(P, P1);
	
	MemPtrFree(P1);	
}

void RemoveBlankHead(char** p){
	while ((**p) == 9 			//Horizontal Tabulation
			|| (**p) == 10		//Line Feed
			|| (**p) == 13		//Carrige Return
			|| (**p) == 32)		//Space
		(*p)++;
}

Boolean DrawField(RectangleType* bounds, char* p, UInt16* top, UInt16 align, 
	UInt16* topAnchor, UInt16* firstLineHeight, Boolean draw){

	Int16 strWidth;
	Int16 strLength;
	Int16 xpos;
	Int16 lineHeight = FntLineHeight();
	char* p2;
	Boolean reachBottom = false;
	
	while (true){
		
		//	remove blank
		RemoveBlankHead(&p);
		
		if ((*p) == 0) break;
		
		//	calculate one line
		strLength = FntWordWrap(p, bounds -> extent.x);
		strWidth = FntCharsWidth(p, strLength);
		
		//	if has a blank tail, cut it
		p2 = p + strLength - 1;
		if ((*p2) == 9 || (*p2) == 10 || (*p2) == 13 || (*p2) == 32)
			strLength--;
		
		//	if the first line crossed by upper bounds, adjust it
		if ((Int16)(*top) < (Int16)bounds->topLeft.y && (Int16)(*top) + lineHeight> bounds->topLeft.y){
			*topAnchor += bounds->topLeft.y - (*top);
			*top = bounds->topLeft.y;
		}
		
		//	if draw and in the bounds area			
		if (draw && (Int16)(*top) >= bounds->topLeft.y){
			
			//	if current line crossed by bottom bounds, dont draw and return reach bottom
			if ((Int16)(*top) + lineHeight > bounds->topLeft.y + bounds->extent.y) {
				reachBottom = true;
				return reachBottom;
			}
			
			//	if it is the first line in bounds, write down the line height for scroll usage
			if ((*top) == bounds->topLeft.y) *firstLineHeight = lineHeight;

			switch (align){
				case leftAlign:
					xpos = bounds -> topLeft.x;
					break;
				case centerAlign:
					xpos = bounds -> topLeft.x + (bounds -> extent.x - strWidth)/2;
					break;
				case rightAlign:
					xpos = bounds -> topLeft.x + bounds -> extent.x - strWidth;
					break;
			}
			WinDrawChars(p, strLength, xpos, *top); 
		}
		
		p += strLength;
		(*top) += lineHeight;
	}
	
	return reachBottom;
}

Boolean DrawFields(RectangleType* bounds, WordFieldFlags flag, UInt16* top, UInt16* topAnchor, 
	UInt16* firstLineHeight, Boolean draw){
	
	FontID currFont = FntGetFont();
	WordRecordType record;
	MemHandle recordH;
	WordFields i;
	SugarAppInfoType * appInfo;
	Boolean reachBottom = false;
	
	*top = *topAnchor;
	
	appInfo = GetAppInfoPtr(CurrentDB);
	GetWordRecord(CurrentDB, CurrentRecord, &record, &recordH);
	
	for (i = WordFieldQuestion; i < WordFieldsCount; i++){
		if (GetBitMacro(flag.allBits, i) != 0 && record.field[i]!= NULL){
			FntSetFont(appInfo -> learnFieldFont[i]);
			
			reachBottom = DrawField(bounds, record.field[i], top, appInfo -> learnFieldAlignment[i], 
				topAnchor, firstLineHeight, draw);
			if (draw && reachBottom) break;
		}
	}
	
	MemHandleUnlock(recordH);
	MemPtrUnlock(appInfo);

	FntSetFont(currFont);
	
	return  reachBottom;
}

Boolean DrawPane(RectangleType* bounds, WordFieldFlags flag, UInt16* topAnchor, UInt16* firstLineHeight){
	
	UInt16 top;	
	
	if (*topAnchor == bounds -> topLeft.y){
		DrawFields(bounds, flag, &top, topAnchor, firstLineHeight, false);
	
		if (top - *topAnchor < bounds -> extent.y) 
			*topAnchor = bounds->topLeft.y + (bounds->extent.y - top + *topAnchor) / 2;
	}
	
	return DrawFields(bounds, flag, &top, topAnchor, firstLineHeight, true);
}

void DrawUpperPane(WordFieldFlags flag){

	RectangleType bounds;
	
	UpperPaneFlag = flag;
	
	GetObjectBounds(Learn2UpperPaneGadget, &bounds);
	
	UpperPaneScrollableDown = DrawPane(&bounds, flag, &UpperPaneTopAnchor, &UpperPaneFirstLineHeight);
	UpperPaneScrollableUp = ((Int16)UpperPaneTopAnchor < bounds.topLeft.y );
	
	if (UpperPaneScrollableUp) ShowObject(Learn2UpperPaneScrollUpRepeating);
	else HideObject(Learn2UpperPaneScrollUpRepeating);
	
	if (UpperPaneScrollableDown) ShowObject(Learn2UpperPaneScrollDownRepeating);
	else HideObject(Learn2UpperPaneScrollDownRepeating);
	
	WinPushDrawState();
	WinSetForeColor(UIColorGetTableEntryIndex(UIObjectSelectedFill));
	WinDrawLine(0, bounds.topLeft.y + bounds.extent.y, 160, bounds.topLeft.y + bounds.extent.y);
	WinPopDrawState();	
}

void DrawLowerPane(WordFieldFlags flag){

	RectangleType bounds;
	
	LowerPaneFlag = flag;
	
	GetObjectBounds(Learn2LowerPaneGadget, &bounds);
	
	LowerPaneScrollableDown = DrawPane(&bounds, flag, &LowerPaneTopAnchor, &LowerPaneFirstLineHeight);
	LowerPaneScrollableUp = ((Int16)LowerPaneTopAnchor < bounds.topLeft.y);
	
	FrmUpdateScrollers(FrmGetActiveForm(), GetObjectIndex(Learn2LowerPaneScrollUpRepeating), 
		GetObjectIndex(Learn2LowerPaneScrollDownRepeating), LowerPaneScrollableUp, LowerPaneScrollableDown);
}

void DrawChoiceTable(){
	
	TableType* table = GetObjectPtr(Learn2ChoiceTable);
	
	RectangleType expandedBounds = {{0, 48}, {160, 112}};
	
	Int16 row;
	UInt16 num, r;
	UInt16 BufferIndex, index;
	HeaderItem item;
	Err error;
	
	SetObjectBounds(Learn2ChoiceTable, &expandedBounds);
	
	switch(HeaderRecordIndex){

		case LearningRecordIndex:
			BufferIndex = LearningChoiceBuffer;
			break;
			
		case TestingRecordIndex:
			BufferIndex = TestingChoiceBuffer;
			break;
			
		default:
			return;	
			break; 
	}
	
	while (FindHeaderItemByDBIndex(CurrentDB, BufferIndex, CurrentRecord, &index))
		RemoveHeaderItem(CurrentDB, BufferIndex, index);
	
	num = GetHeaderItemNumber(CurrentDB, BufferIndex);
	
	ShuffleHeaderItem(CurrentDB, BufferIndex);

	for(row = 0; row < num && row < RowsInChoiceTable - 1; row++){
		
		GetHeaderItem(CurrentDB, BufferIndex, &item, row);
		
		error = DmFindRecordByID(CurrentDB, item, &index);
		if (error) {
			ErrAlert(error);
			index = noRecord;
			continue;
		}
		
		TblSetRowID(table, row, index);
		TblSetRowUsable(table, row, true);
	}
	
	if (row != 0){
		r = SysRandom(0);
		if (r == sysRandomMax) r--;
		index = (UInt32)r * (row + 1) / sysRandomMax;
		
		if (index < row)
			TblSetRowID(table, row, TblGetRowID(table, (Int16)index));
		
		TblSetRowID(table, index, CurrentRecord);
	}
	else 
		TblSetRowID(table, row, CurrentRecord);
	
	TblSetRowUsable(table, row, true);
	row ++;
		
	while (row < RowsInChoiceTable){
		TblSetRowUsable(table, row, false);
		row ++;
	}
	
	TblDrawTable(table);
	
	StartingTicks = TimGetTicks();
	
	ShowChoices = true;
}

void Learn2ScrollUpperPane(Boolean up){
	
	ErasePane(Learn2UpperPaneGadget);
	
	if (up){
		if (UpperPaneScrollableUp == false) return;
		UpperPaneTopAnchor ++; 
		DrawUpperPane(UpperPaneFlag);
	}
	else{
		if (UpperPaneScrollableDown == false) return;
		UpperPaneTopAnchor -= UpperPaneFirstLineHeight;
		DrawUpperPane(UpperPaneFlag);
	}
}

void Learn2ScrollLowerPane(Boolean up){
	
	ErasePane(Learn2LowerPaneGadget);
	
	if (up){
		if (LowerPaneScrollableUp == false) return;
		LowerPaneTopAnchor ++;
		DrawLowerPane(LowerPaneFlag);
	}
	else{
		if (LowerPaneScrollableDown == false) return;
		LowerPaneTopAnchor -= LowerPaneFirstLineHeight;
		DrawLowerPane(LowerPaneFlag);
	}
}

Boolean LoadWord(){

	Err error;
	HeaderItem item;
	
	while(true) {
		if (0 == GetHeaderItemNumber(CurrentDB, HeaderRecordIndex)){
			//Finish();
			return false;
		}
		
		GetHeaderItem(CurrentDB, HeaderRecordIndex, &item, 0); //fetch first item
		error = DmFindRecordByID(CurrentDB, item, &CurrentRecord);
		
		if (error) {
			ErrAlert(error);
			RemoveHeaderItem(CurrentDB, HeaderRecordIndex, 0);
			continue;
		}
		
		//	if learning, its done
		if (HeaderRecordIndex == LearningRecordIndex) break;
		
		//	if testing, and if not finished
		if (TossNextReviewMode() != 0) break;
	
		Learn2FinishReview();
	}

	return true;
}

void Learn2InitWord(){

	
	Learn2ClipBoardWord();
	//reset all interface settings
	ResetUpperPane();
	ResetLowerPane();

	if (HeaderRecordIndex == LearningRecordIndex)
		CurrentLearningStatus = GetLearningStatus(CurrentDB, CurrentRecord);
	
	if (HeaderRecordIndex == TestingRecordIndex)
		CurrentLearningStatus = TossNextReviewMode();
	
	switch(CurrentLearningStatus){
		case FirstLook:
			DrawTranslationLook();
			break;
	
		case TranslationRecollection:
			DrawTranslationRecollection();
			break;
		
		case WordRecollection:
			DrawWordRecollection();
			break;
			
		case TranslationTest:
			DrawTranslationTest();
			StartingTicks = TimGetTicks();
			break;
			
		case WordTest:
			DrawWordTest();
			StartingTicks = TimGetTicks();
			break;
	}
}

void DrawWordLook(){

	WordFieldFlags flag;

	flag.allBits = 0;
	SetBitMacro(flag.allBits, WordFieldAnswer);
	DrawUpperPane(flag);
		
	DefaultField = WordFieldQuestion;
	LoadLowerPaneTab();
				
	flag.allBits = 0;
	SetBitMacro(flag.allBits, DefaultField);
	DrawLowerPane(flag);
				
	ShowObject(Learn2NextButton);
	ShowNext = true;
}

void DrawTranslationLook(){
	WordFieldFlags flag;

	flag.allBits = 0;
	SetBitMacro(flag.allBits, WordFieldQuestion);
	SetBitMacro(flag.allBits, WordFieldPhonetic);
	DrawUpperPane(flag);
		
	DefaultField = WordFieldAnswer;
	LoadLowerPaneTab();
				
	flag.allBits = 0;
	SetBitMacro(flag.allBits, DefaultField);
	DrawLowerPane(flag);
				
	ShowObject(Learn2NextButton);
	ShowNext = true;
}

void DrawTranslationRecollection(){
	
	WordFieldFlags flag;

	flag.allBits = 0;
	SetBitMacro(flag.allBits, WordFieldQuestion);
	SetBitMacro(flag.allBits, WordFieldPhonetic);
	DrawUpperPane(flag);
				
	ShowObject(Learn2ShowButton);
	ShowShow = true;
}
void DrawWordRecollection(){

	WordFieldFlags flag;
	
	flag.allBits = 0;
	SetBitMacro(flag.allBits, WordFieldAnswer);
	DrawUpperPane(flag);
				
	ShowObject(Learn2ShowButton);
	ShowShow = true;
}

void DrawTranslationTest(){

	WordFieldFlags flag;

	flag.allBits = 0;
	SetBitMacro(flag.allBits, WordFieldQuestion);
	SetBitMacro(flag.allBits, WordFieldPhonetic);
	DrawUpperPane(flag);
				
	ChoiceField = WordFieldAnswer;
	DrawChoiceTable();
}

void DrawWordTest(){
	WordFieldFlags flag;
	
	flag.allBits = 0;
	SetBitMacro(flag.allBits, WordFieldAnswer);
	DrawUpperPane(flag);
				
	ChoiceField = WordFieldQuestion;
	DrawChoiceTable();
}

void ResetUpperPane(){

	RectangleType bounds;

	GetObjectBounds(Learn2UpperPaneGadget, &bounds);
	UpperPaneTopAnchor = bounds.topLeft.y;
	UpperPaneFirstLineHeight = 0;
	UpperPaneScrollableUp = false;
	UpperPaneScrollableDown = false;
	
	HideObject(Learn2UpperPaneScrollUpRepeating);
	HideObject(Learn2UpperPaneScrollDownRepeating);
}

void ResetLowerPane(){

	RectangleType bounds;

	GetObjectBounds(Learn2LowerPaneGadget, &bounds);
	LowerPaneTopAnchor = bounds.topLeft.y;
	LowerPaneFirstLineHeight = 0;
	LowerPaneScrollableUp = false;
	LowerPaneScrollableDown = false;
	FrmUpdateScrollers(FrmGetActiveForm(), GetObjectIndex(Learn2LowerPaneScrollUpRepeating), 
		GetObjectIndex(Learn2LowerPaneScrollDownRepeating), false, false);

}

WordFieldFlags GetQuestionFields(){
	WordFieldFlags flag;
	SugarAppInfoType* app;
	app = GetAppInfoPtr(CurrentDB);
	flag = app->questionPageFields;
	MemPtrUnlock(app);
	return flag;	
}

WordFieldFlags GetAnswerFields(){
	WordFieldFlags flag;
	SugarAppInfoType* app;
	app = GetAppInfoPtr(CurrentDB);
	flag = app->answerPageFields;
	MemPtrUnlock(app);
	return flag;	
}

void LoadLowerPaneTab(){
	
	FormType* frmP;
	FormType** formPP;
	SugarAppInfoType* app;
	WordRecordType record;
	MemHandle recordH;
	UInt16 xpos, ypos = 147, width = 12, height = 12;
	WordFields i;	
	Boolean lefty = Lefty();
	
	if (lefty) xpos = 146;
	else xpos = 1;
	
	app = GetAppInfoPtr(CurrentDB);
	GetWordRecord(CurrentDB, CurrentRecord, &record, &recordH);
	
	for (i = WordFieldQuestion; i < WordFieldsCount; i++){
		
		if (i == DefaultField || (GetBitMacro(app -> answerPageFields.allBits, i) != 0) 
			&& (record.field[i] != NULL)){
			
			frmP = FrmGetActiveForm();
			formPP = &frmP;
			FieldTab[i] = CtlNewControl((void**)formPP, DynamicPushButtonStart + i, pushButtonCtl, 
				fieldLabel[i], xpos, ypos, width, height, stdFont, DynamicPushButtonGroup, false);
			
			CtlDrawControl(FieldTab[i]);
			if (lefty) xpos -= width + 1;
			else xpos += width + 1;			
		}
	
	}
	MemPtrUnlock(app);
	MemHandleUnlock(recordH);
	
	FrmSetControlGroupSelection(FrmGetActiveForm(), DynamicPushButtonGroup, DynamicPushButtonStart + DefaultField);
}

void ClearLowerPaneTab(){
	
	FormType* frmP;
	FormType** formPP;
	
	UInt16 index;
	WordFields i;
	
	SugarAppInfoType* app;
	WordRecordType record;
	MemHandle recordH;
	
	app = GetAppInfoPtr(CurrentDB);
	GetWordRecord(CurrentDB, CurrentRecord, &record, &recordH);
	
	for (i = WordFieldQuestion; i < WordFieldsCount; i++){
		if (i == DefaultField	|| (GetBitMacro(app -> answerPageFields.allBits, i) != 0) 
			&& (record.field[i] != NULL)){
		
			frmP = FrmGetActiveForm();
			index = FrmGetObjectIndex(frmP, DynamicPushButtonStart + i);
		
			if (index == frmInvalidObjectId) continue;

			HideObject(DynamicPushButtonStart + i);
		
			formPP = &frmP;
			FrmRemoveObject(formPP, index);
		}
	
	}
	
	MemPtrUnlock(app);
	MemHandleUnlock(recordH);
}

void ErasePane(UInt16 ObjectID){
	RectangleType bounds;
	
	GetObjectBounds(ObjectID, &bounds);
	WinEraseRectangle(&bounds, 0);
}


void Learn2FormDoButtonNext(){

	Boolean right = false;

	//	clear up
	ClearLowerPaneTab();
	ErasePane(Learn2UpperPaneGadget);
	ErasePane(Learn2LowerPaneGadget);
	HideObject(Learn2NextButton);
	ShowNext = false;
	
	if (HeaderRecordIndex == LearningRecordIndex){

		if (CurrentLearningStatus == FirstLook) right = true;
		Learn2ProcessLearningRating(right);

	}
	
	if (HeaderRecordIndex == TestingRecordIndex){
		Learn2ProcessReviewRating(FailRating);
	}	
}

void Learn2FormDoButtonShow(){

	WordFieldFlags flag;
	
	HideObject(Learn2ShowButton);
	ShowShow = false;
	
	if (HeaderRecordIndex == LearningRecordIndex){
		switch(CurrentLearningStatus){
			case TranslationRecollection:
				DefaultField = WordFieldAnswer;
				break;
				
			case WordRecollection:
				
				DefaultField = WordFieldQuestion;
				break;
		}

		ShowObject(Learn2RightPushButton);
		ShowObject(Learn2WrongPushButton);
		FrmSetControlGroupSelection(FrmGetActiveForm(), RightWrongGroup, Learn2RightPushButton);
		ShowRightWrong = true;
	}
	
	if (HeaderRecordIndex == TestingRecordIndex){
		switch(CurrentLearningStatus){
			case TranslationRecollection:
				DefaultField = WordFieldAnswer;
				break;
				
			case WordRecollection:
				
				DefaultField = WordFieldQuestion;
				break;
		}
		ShowObject(Learn2BingoPushButton);
		ShowObject(Learn2PassPushButton);
		ShowObject(Learn2FailPushButton);
		FrmSetControlGroupSelection(FrmGetActiveForm(), BingoPassFailGroup, Learn2BingoPushButton);
		ShowBingoPassFail = true;
	}		
	
	ResetLowerPane();
	LoadLowerPaneTab();				
	flag.allBits = 0;
	SetBitMacro(flag.allBits, DefaultField);
	DrawLowerPane(flag);
}

void Learn2FormDoButtonPush(UInt16 objectID){

	Boolean right;
	Rating rating;

	ErasePane(Learn2UpperPaneGadget);
	ErasePane(Learn2LowerPaneGadget);
	ClearLowerPaneTab();
	ResetAllPush();

	if (HeaderRecordIndex == LearningRecordIndex){
		if (objectID == Learn2RightPushButton) right = true;
		else right = false;

		Learn2ProcessLearningRating(right);	
	}
	
	if (HeaderRecordIndex == TestingRecordIndex){
		
		switch(objectID){
			case	Learn2BingoPushButton:
				rating = BingoRating;
				break;
			case 	Learn2PassPushButton:
				rating = PassRating;
				break;
			case	Learn2FailPushButton:
				rating = FailRating;
				break;
		}
		
		Learn2ProcessReviewRating(rating);
	}
}

void Learn2DoDynamicPushButton(WordFields field){
	
	WordFieldFlags flag;
	
	ErasePane(Learn2LowerPaneGadget);
	ResetLowerPane();
	
	flag.allBits = BitAtPosition(field);
	DrawLowerPane(flag);
	
}


void ResetAllPush(){
	
	ControlType* control;
	
	if (HeaderRecordIndex == LearningRecordIndex){
		control = GetObjectPtr(Learn2RightPushButton);
		CtlSetValue(control, 0);
		control = GetObjectPtr(Learn2WrongPushButton);
		CtlSetValue(control, 0);
		
		HideObject(Learn2RightPushButton);
		HideObject(Learn2WrongPushButton);
		ShowRightWrong = false;
	}
	
	if (HeaderRecordIndex == TestingRecordIndex){
		control = GetObjectPtr(Learn2BingoPushButton);
		CtlSetValue(control, 0);
		control = GetObjectPtr(Learn2PassPushButton);
		CtlSetValue(control, 0);
		control = GetObjectPtr(Learn2FailPushButton);
		CtlSetValue(control, 0);		
		
		HideObject(Learn2BingoPushButton);
		HideObject(Learn2PassPushButton);
		HideObject(Learn2FailPushButton);
		ShowBingoPassFail = false;
	}
}

void Learn2FormTableSelection(Int16 row){
	
	TableType* table = GetObjectPtr(Learn2ChoiceTable);
	Rating	rating = NoRating;
	MemorizingStatus ms = GetMemorizingStatus(CurrentDB, CurrentRecord);
	
	TblUnhighlightSelection(table);
	
	ErasePane(Learn2UpperPaneGadget);
	ClearChoiceTable();
	
	
	if (HeaderRecordIndex == LearningRecordIndex){		

		if (TblGetRowID(table, row) == CurrentRecord) {
			Learn2ProcessLearningRating(true);
			return;
		} 

		if (CurrentLearningStatus == TranslationTest) DrawTranslationLook();
		if (CurrentLearningStatus == WordTest) DrawWordLook();
		return;
	}
	
	if (HeaderRecordIndex == TestingRecordIndex){
		
		if (TblGetRowID(table, row) == CurrentRecord){
			if (TimGetTicks() - StartingTicks < 6 * SysTicksPerSecond()) rating = BingoRating;
			else rating = PassRating;
			
			Learn2ProcessReviewRating(rating);
			return;
		}
		
		if (CurrentLearningStatus == TranslationTest) DrawTranslationLook();
		if (CurrentLearningStatus == WordTest) DrawWordLook();
		return;
	}
}

void ClearChoiceTable(){
	TableType* table = GetObjectPtr(Learn2ChoiceTable);
	RectangleType bounds;
	RectangleType compactBounds = {{0, 48}, {160, 97}};
	
	Int16 row;
	
	for (row = 0; row < RowsInChoiceTable; row ++){
		TblSetRowUsable(table, row, false);
	}
	
	GetObjectBounds(Learn2ChoiceTable, &bounds);
	WinEraseRectangle(&bounds, 0);
	
	SetObjectBounds(Learn2ChoiceTable, &compactBounds);
	ShowChoices = false;
}

void Learn2FormQuit(){
	if (CurrentDB != NULL && CurrentDB != 0)
		DmCloseDatabase(CurrentDB);
	
	HeaderRecordIndex = HeaderIndexCounts;
	FrmGotoForm(MainForm);
}

LearningStatus TossNextReviewMode(){
	
	SugarAppInfoType* app;
	ReviewMode reviewMode;
	TestingStatus ts = GetTestingStatus(CurrentDB, CurrentRecord);
	UInt16 i = 0, j = 0, r = 0, r0 = 0;
	
	app = GetAppInfoPtr(CurrentDB);
	reviewMode = app -> reviewMode;
	MemPtrUnlock(app);
	
	r = (UInt32)SysRandom(0) * 12 /sysRandomMax;
	r0 = r;
	i = TranslationRecollection;
	
	while (true) {

		if (GetBitMacro((UInt16)(reviewMode.allBits), i)!= 0 && GetBitMacro((UInt16)(ts.reviewMode.allBits), i) == 0 ){
			if (r == 0) return i;
			else r--;
		}
		
		if (i == 4 && r == r0) return 0; // finished
		if (i == 4) i = 1;
		else i ++;
	}
	
	return 0;	
}

void Learn2FinishReview(){
	
	Char rate, strErrTxt[100];
	Rating r1, r2;
//	Rating maxRating;
	TestingStatus ts = GetTestingStatus(CurrentDB, CurrentRecord);
	MemorizingStatus ms = GetMemorizingStatus(CurrentDB, CurrentRecord);
	UInt16 interval, tFail;
	DateType today;
	
	r1 = ts.firstRating;
	r2 = ts.secondRating;
	tFail = ts.totalFail;
	
	DateSecondsToDate(TimGetSeconds(), &today);
	
/*	if (r1 < r2) {
		maxRating = r1;
		
		// Added by hytown
		if (maxRating == NoRating) maxRating = r2;
		}
	else {
		maxRating = r2;
	
		// Added by hytown
		if (maxRating == NoRating) maxRating = r1;
	}*/
		
	//	double ace
	if ((r1 == BingoRating && r2 == BingoRating) || (r1 == BingoRating && r2 == NoRating)) rate = 'A';
	
	//	first ace, second none, or pass
//	else if (r1 == BingoRating && r2 <= PassRating) rate = 'B';
	// Changed by hytown
//	else if (maxRating == BingoRating) rate = 'B';
	else if ((r1 == BingoRating && r2 == PassRating) \
		|| (r1 == PassRating && r2 == BingoRating) \
		|| (r1 == FailRating && r2 == BingoRating && tFail == 1) \
		|| (r1 == PassRating && r2 == NoRating)) rate = 'B';
	
	//	first pass, second bingo or pass
//	else if (maxRating == PassRating) rate = 'C';
	// Changed by hytown
	else if ((r1 == FailRating && r2 == PassRating && tFail == 1) \
		|| (r1 == FailRating && r2 == BingoRating && tFail == 2) \
		|| (r1 == PassRating && r2 == PassRating)) rate = 'C';
	
	//	at least one fail, only fail once
//	else if (maxRating == FailRating && ts.totalFail == 1) rate = 'D';
	// Changed by hytown
	else if ((r1 == FailRating && r2 == PassRating && tFail == 2) \
		|| (r1 == FailRating && r2 == BingoRating && tFail > 2 && tFail < maxFail)) rate = 'D';
	
	//	fail many times
//	else if (maxRating == FailRating && ts.totalFail < maxFail) rate = 'E';
	// Changed by hytown
	else if (r1 == FailRating && r2 == PassRating && tFail > 2 && tFail < maxFail) rate = 'E';

	//	fail too many times
	else if (tFail >= maxFail) rate = 'F';
	
	// Added by hytown
	// Some situation not expected
	else {
		rate = 'F';
		StrPrintF(strErrTxt, "%s%i%s%i%s%i%s", "The situation is unexpected. firstRating = ", r1, \
			"; secondRating = ", r2, "; totalFail = ", tFail, ". Please report this error message on hi-pda.com, thanks.");
		DoCustomDialog("Error", strErrTxt, false, NULL, NULL, true);
	}
		
	/*
	float 		eFactor;		//E-Factor
	UInt8 		difficulty;		//
	UInt8 		totalLapse;	
	UInt8 		totalRecall;
	UInt8 		step;			//step in current round
	DateType 	lastTest;
	DateType	currentTest;	//when the current test begins, only meaningful for drilling words
	DateType 	nextTest;
	DateType 	firstLearn;
	DateType 	create;
	*/
	
	switch (rate){
		case 'A':
			if (ms.step == 1) {
				ms.totalRecall++;
				ms.step++;
				
				if (DateToDays(ms.nextTest) - DateToDays(ms.lastTest) > 2)
					interval = (Int32)((ms.eFactor - 1.3) * 8 / 1.2) + 13;
				else if (DateToDays(ms.nextTest) - DateToDays(ms.lastTest) == 2)
					interval = 7;
				else
					interval = 6;
					
				ms.lastTest = today;
				ms.nextTest = today;					 
				DateAdjust(&ms.nextTest, interval);
			} 
			else {
				ms.eFactor += 0.1;
				ms.difficulty = Difficulty(ms.eFactor);
				ms.totalRecall++;
				ms.step++;
				interval = DateToDays(ms.nextTest) - DateToDays(ms.lastTest);
				ms.lastTest = today;
				ms.nextTest = today;
				DateAdjust(&ms.nextTest, (UInt16)(interval * ms.eFactor));
			}
			break;
			 	
		case 'B':
			if (ms.step == 1) {
				ms.totalRecall++;
				ms.step++;

				if (DateToDays(ms.nextTest) - DateToDays(ms.lastTest) > 2)
					interval = (Int32)((ms.eFactor - 1.3) * 8 / 1.2) + 13;
				else if (DateToDays(ms.nextTest) - DateToDays(ms.lastTest) == 2)
					interval = 6;
				else
					interval = 5;

				ms.lastTest = today;
				ms.nextTest = today;
				DateAdjust(&ms.nextTest, interval);
			} 
			else {
				ms.difficulty = Difficulty(ms.eFactor);
				ms.totalRecall++;
				ms.step++;
				interval = DateToDays(ms.nextTest) - DateToDays(ms.lastTest);
				ms.lastTest = today;
				ms.nextTest = today;
				DateAdjust(&ms.nextTest, (UInt16)(interval * ms.eFactor));
			}
			break;
			
		case 'C':
			if (ms.step == 1) {
				ms.totalRecall++;
				ms.step++;

				if (DateToDays(ms.nextTest) - DateToDays(ms.lastTest) > 2)
					interval = (Int32)((ms.eFactor - 1.3) * 8 / 1.2) + 13;
				else if (DateToDays(ms.nextTest) - DateToDays(ms.lastTest) == 2)
					interval = 5;
				else
					interval = 4;
				
				ms.lastTest = today;
				ms.nextTest = today;
				DateAdjust(&ms.nextTest, interval);
			} 
			else {
				ms.eFactor -= 0.14;
				if (ms.eFactor < 1.3) ms.eFactor = 1.3;
				
				ms.difficulty = Difficulty(ms.eFactor);
				ms.totalRecall++;
				ms.step++;
				interval = DateToDays(ms.nextTest) - DateToDays(ms.lastTest);
				ms.lastTest = today;
				ms.nextTest = today;
				DateAdjust(&ms.nextTest, (UInt16)(interval * ms.eFactor));
			}
			break;
			
		case 'D':
			ms.eFactor -= 0.32;
			if (ms.eFactor < 1.3) ms.eFactor = 1.3;
			
			ms.difficulty = Difficulty(ms.eFactor);
			ms.totalLapse++;
			ms.step = 1;
			interval = 8 - (Int32)((ms.eFactor - 1.3) * 3 / 1.2);
			ms.lastTest = today;
			ms.nextTest = today;
			DateAdjust(&ms.nextTest, interval);
			break;
			
		case 'E':
			ms.eFactor -= 0.54;
			if (ms.eFactor < 1.3) ms.eFactor = 1.3;
			
			ms.difficulty = Difficulty(ms.eFactor);
			ms.totalLapse++;
			ms.step = 1;
			interval = 8 - (Int32)((ms.eFactor - 1.3) * 3 / 1.2);
			ms.lastTest = today;
			ms.nextTest = today;
			DateAdjust(&ms.nextTest, interval);
			break;
					
		case 'F':
			ms.eFactor -= 0.8;
			if (ms.eFactor < 1.3) ms.eFactor = 1.3;
			
			ms.difficulty = Difficulty(ms.eFactor);
			ms.totalLapse++;
			ms.step = 0;
			ms.lastTest = today;
			DateSecondsToDate(0, &ms.nextTest);
			break;
	}
	
	//	clear testing status!
	
	// Added by hytown
	ts.firstRating = NoRating;
	ts.secondRating = NoRating;
	ts.totalFail = 0;
	SetTestingStatus(CurrentDB, CurrentRecord, ts);
	
	SetLearningStatus(CurrentDB, CurrentRecord, FirstLook);
	SetMemorizingStatus(CurrentDB, CurrentRecord, ms);
	
	if (rate == 'F')
		SetQueue(CurrentDB, CurrentRecord, LearningQueue);
	else
		SetQueue(CurrentDB, CurrentRecord, WaitingQueue);
}

void Learn2FinishLearning(){

	MemorizingStatus ms = GetMemorizingStatus(CurrentDB, CurrentRecord);

	SetLearningStatus(CurrentDB, CurrentRecord, FirstLook);
	
	/*
	float 		eFactor;		//E-Factor
	UInt8 		difficulty;		//
	UInt8 		totalLapse;	
	UInt8 		totalRecall;
	UInt8 		step;			//step in current round
	DateType 	lastTest;
	DateType	currentTest;	//when the current test begins, only meaningful for drilling words
	DateType 	nextTest;
	DateType 	firstLearn;
	DateType 	create;
	*/
	
	ms.step = 1;
	DateSecondsToDate(TimGetSeconds(), &ms.lastTest);
	ms.nextTest = ms.lastTest;
	
	if (ZeroDate(ms.firstLearn))	{
		ms.firstLearn = ms.nextTest;
		DateAdjust(&ms.nextTest, 1);
	}
	else
		DateAdjust(&ms.nextTest, 2);
		
	SetMemorizingStatus(CurrentDB, CurrentRecord, ms);
	SetQueue(CurrentDB, CurrentRecord, WaitingQueue);
}

UInt8 Difficulty(float efactor){

	UInt8 x = (UInt8)((efactor - 1.3)*50);
	if (x > 100) return (UInt8) 100;
	return (UInt8)(100 - x);

}

void Learn2ProcessLearningRating(Boolean right){
	
	UInt16 step;
	
	PushHeaderBuffer(CurrentDB, LearningChoiceBuffer, CurrentRecord);
	
	switch(CurrentLearningStatus){
		case FirstLook:
			step = 6;
			break;
			
		case TranslationRecollection:
			step = 13;
			break;
			
		case WordRecollection:
			step = 27;
			break;
			
		case TranslationTest:
			step = 55;
			break;
	
		case WordTest:
			step = 111;
			break;
	}
		
	if (right) {
		if (CurrentLearningStatus == WordTest)
			Learn2FinishLearning();
			
		else {
			CurrentLearningStatus++;
			SetLearningStatus(CurrentDB, CurrentRecord, CurrentLearningStatus);
			MoveHeaderItem(CurrentDB, LearningRecordIndex, step);
		}	
	}
	else
		MoveHeaderItem(CurrentDB, LearningRecordIndex, step);

	if (LoadWord()) Learn2InitWord();
	else Learn2FormQuit();
	return;			
}

void Learn2ProcessReviewRating(Rating rating){

	TestingStatus ts = GetTestingStatus(CurrentDB, CurrentRecord);
	UInt16 bits;	
	
	//	Set rating status
	if (ts.firstRating == NoRating) ts.firstRating = rating;
	else if (ts.secondRating < rating) ts.secondRating = rating;

	// I changed the statement back to original since I have changed the enum Rating
	// Changed by hytown
//	else if ((ts.secondRating == NoRating) || (ts.secondRating > rating)) ts.secondRating = rating;
	
	if (rating == FailRating) ts.totalFail++;
	
	SetTestingStatus(CurrentDB, CurrentRecord, ts);

	PushHeaderBuffer(CurrentDB, TestingChoiceBuffer, CurrentRecord);				
	
	//	if bingo
	if (rating == BingoRating) {
		
		//	finish a review mode, set correspondent bit		
		/*
		bits = (UInt16)ts.reviewMode.allBits;
		SetBitMacro(bits, CurrentLearningStatus);
		ts.reviewMode.allBits = (UInt8)bits;
		SetTestingStatus(CurrentDB, CurrentRecord, ts);
		
		if (TossNextReviewMode() == 0) Learn2FinishReview();
		else MoveHeaderItem(CurrentDB, TestingRecordIndex, 55);
		*/
		
		Learn2FinishReview();
	}
	
	//	if pass
	else if (rating == PassRating){

		bits = (UInt16)ts.reviewMode.allBits;
		SetBitMacro(bits, CurrentLearningStatus);
		ts.reviewMode.allBits = (UInt8)bits;
		SetTestingStatus(CurrentDB, CurrentRecord, ts);
		
		if (TossNextReviewMode() == 0) Learn2FinishReview();
		else MoveHeaderItem(CurrentDB, TestingRecordIndex, 55);	
	}
	// if fail
	else
		MoveHeaderItem(CurrentDB, TestingRecordIndex, 55);

	if (LoadWord()) Learn2InitWord();
	else Learn2FormQuit();
	return;			
}

Boolean Learn2FormDoHardKey(EventType* event){
	
	Boolean selectChoice = false;
	Boolean hitNext = false;
	Boolean hitShow = false;
	Boolean nextTab = false;
	Boolean prevTab = false;
	Boolean toggleRate = false;
	Boolean pushRate = false;
	Boolean prevChoice = false;
	Boolean nextChoice = false;
	
	Boolean handled = false;
	Boolean lefty = Lefty();
	
	ControlType* ctl;
	TableType* table;
	Int16 row, column;
	FormType* form = FrmGetActiveForm();
	UInt16 ObjIndex;
	
	WChar key = event->data.keyDown.chr;
	
	SndPlaySystemSound(sndClick);
	
	if (key == hard1Chr || key == hard2Chr || key == hard3Chr || key == hard4Chr) handled = true; 
	if ((key <= vchrSonyMin) && (key >= vchrSonyMax)) handled = true;

	if (ShowNext){
		if (((key == hard2Chr) && lefty) || ((key == hard3Chr) && (!lefty)))
			hitNext = true;
		
		//	Sony	
		if ((key == vchrJogPush) && lefty)
			hitNext = true;
	}
	
	if (ShowShow){
		if ((key == hard2Chr && lefty) || (key == hard3Chr && !lefty)) 
			hitShow = true;
		
		//	Sony	
		if ((key == vchrJogPush) && lefty)
			hitShow = true;	
	}
	
	if (ShowRightWrong || ShowBingoPassFail){
		if ((key == hard1Chr && lefty) || (key == hard4Chr && !lefty))
			toggleRate = true;
			
		if ((key == hard2Chr && lefty) || (key == hard3Chr && !lefty))
			pushRate = true;
		
		//	Sony	
		if (lefty && (key == vchrJogDown))
			toggleRate = true;
		
		if (lefty && (key == vchrJogPush))
			pushRate = true;
	}
	
	if (ShowNext || ShowRightWrong || ShowBingoPassFail){
	
		if (key == hard1Chr && !lefty) prevTab = true;
		if (key == hard2Chr && !lefty) nextTab = true;
		if (key == hard3Chr && lefty) nextTab = true;
		if (key == hard4Chr && lefty) prevTab = true;
		
		//	Sony
		if (lefty && (key == vchrJogUp)) nextTab = true;
	}
	
	if (ShowChoices){
		if (key == hard1Chr || key == hard2Chr || key == hard3Chr || key == hard4Chr)
			selectChoice = true;
		
		//	Sony
		if (lefty && (key == vchrJogPush))
			selectChoice = true;
			
		if (lefty && (key == vchrJogUp))
			prevChoice = true;
			
		if (lefty && (key == vchrJogDown))
			nextChoice = true;
	}
	
	if (hitNext){
		ctl = GetObjectPtr(Learn2NextButton);
		CtlHitControl(ctl);
	}
	
	if (hitShow){
		// Added by hytown
		// Add some code to avoid crashing
		if (FrmGetActiveFormID() != EditForm) {
		
		ctl = GetObjectPtr(Learn2ShowButton);
		CtlHitControl(ctl);
		}
	}
	
	if (toggleRate){
		if (ShowRightWrong){
		
			ObjIndex = FrmGetControlGroupSelection(form, RightWrongGroup);
			
			if (ObjIndex == GetObjectIndex(Learn2RightPushButton))
				ObjIndex = Learn2WrongPushButton;
			else 
				ObjIndex = Learn2RightPushButton;
				
			FrmSetControlGroupSelection(form, RightWrongGroup, ObjIndex);
		}
		else if (ShowBingoPassFail){

			ObjIndex = FrmGetControlGroupSelection(form, BingoPassFailGroup);
			
			if (ObjIndex == GetObjectIndex(Learn2BingoPushButton))
				ObjIndex = Learn2PassPushButton;
			else if (ObjIndex == GetObjectIndex(Learn2PassPushButton))
				ObjIndex = Learn2FailPushButton;
			else 
				ObjIndex = Learn2BingoPushButton;
				
			FrmSetControlGroupSelection(form, BingoPassFailGroup, ObjIndex);
		}
	}
	
	if (pushRate){
		
		if (ShowRightWrong)
			ObjIndex = FrmGetControlGroupSelection(form, RightWrongGroup);
		if (ShowBingoPassFail)
			ObjIndex = FrmGetControlGroupSelection(form, BingoPassFailGroup);
		
		if (ObjIndex != frmNoSelectedControl){
			ctl = FrmGetObjectPtr(form, ObjIndex);
			CtlHitControl(ctl);
		}
	}
	
	if (prevTab) ToggleDynamicField(true);
	if (nextTab) ToggleDynamicField(false);
	
	if (selectChoice) {
		
		table = GetObjectPtr(Learn2ChoiceTable);
		if (TblGetSelection(table, &row, &column))
			Learn2FormTableSelection(row);
	}
	
	if (prevChoice)	Learn2ScrollChoiceSelection(true);
	if (nextChoice) Learn2ScrollChoiceSelection(false);
	
	return handled;
}

void ToggleDynamicField(Boolean prevField){
	
	FormType* form = FrmGetActiveForm();
	
	WordRecordType record;
	MemHandle recordH;
	
	SugarAppInfoType* app;
	ControlType* ctl;
	
	UInt16 objectID, controlID[WordFieldsCount];
	UInt16 i, j, k, result;
	
	objectID = (UInt16)FrmGetControlGroupSelection(form, DynamicPushButtonGroup);
		
	if (objectID == frmNoSelectedControl) return;
	
	//	Get record
	app = GetAppInfoPtr(CurrentDB);
	GetWordRecord(CurrentDB, CurrentRecord, &record, &recordH);
	
	//	Get available tab list
	j = 0;
	for (i = WordFieldQuestion; i < WordFieldsCount; i++){
		
		if (i == DefaultField || (GetBitMacro(app -> answerPageFields.allBits, i) != 0) 
			&& (record.field[i] != NULL)){
		
			controlID[j] = DynamicPushButtonStart + i;
			j++;
			
		}
	
	}
	
	MemPtrUnlock(app);
	MemHandleUnlock(recordH);
	
	// 	Find selected tab
	for (k = 0; k < j; k++){
		if (FrmGetObjectIndex(form, controlID[k]) == objectID) {
			break;
		}
	}
	
	if (k == j) return;
	
	if (prevField){
		if (k == 0)	result = controlID[j-1];
		else result = controlID[k - 1];
	}
	else {
		if (k == j - 1) result = controlID[0];
		else result = controlID[k + 1];
	}
	
	FrmSetControlGroupSelection(form, DynamicPushButtonGroup, result);
	ctl = GetObjectPtr(result);	
	CtlHitControl(ctl);
}


Boolean Lefty(){
	
	SugarAppInfoType* app;
	Boolean lefty;
	
	app = GetAppInfoPtr(CurrentDB);
	
	if (app == NULL){
		ErrNonFatalDisplayIf(true, "cannot get lefty info");
		return false;
	}
	
	if (app -> lefty == 0) lefty = false;
	else lefty = true;
	
	MemPtrUnlock(app);
	
	return lefty;
}

void Learn2ScrollChoiceSelection(Boolean up){

	TableType* table = GetObjectPtr(Learn2ChoiceTable);
	Int16 row, column;
	Int16 bottom;
		
	if (!TblGetSelection(table, &row, &column)) {
		
		row = TblGetLastUsableRow(table);
		row = row / 2;
		TblSelectItem(table, row, 0);
	
		return;
	}
	
	bottom = TblGetLastUsableRow(table);

	if (up){
		if (row == 0) row = bottom;
		else row --;
	}
	else{
		if (row == bottom) row = 0;
		else row ++;
	}
	
	TblSelectItem(table, row, 0);
}

void Learn2ClipBoardWord(){

	WordRecordType record;
	MemHandle recordH;
	
	GetWordRecord(CurrentDB, CurrentRecord, &record, &recordH);
	
	if (record.field[WordFieldQuestion] != NULL){
	
		ClipboardAddItem(clipboardText, record.field[WordFieldQuestion], StrLen(record.field[WordFieldQuestion]));
	
	}
	
	MemHandleUnlock(recordH);
	return;
}